@@ -637,7 +637,7 @@ System.out.println("pad value="+longitude.getPadValue()+" status="+cdf.getStatus
           double epData  = Epoch.compute(1999, 3, 5, 5, 6, 7, 100),
                  epData1 = Epoch.compute(1998, 1, 2, 3, 4, 5, 666);
           String e0 = Epoch.encode(epData1);
-          double p0 = Epoch.parse(e0);
+          double p0 = Epoch.toParse(e0);
 
           ep.putSingleData(0L, new long[] {0}, new Double(epData)); 
           ep.putSingleData(1L, new long[] {0}, new Double(p0)); 
@@ -703,7 +703,7 @@ System.out.println("pad value="+longitude.getPadValue()+" status="+cdf.getStatus
           long [] nanosecs = new long[5];
           long nansecSinceJ2000;
           try {
-            nansecSinceJ2000 = CDFTT2000.fromUTCstring(dateTime);
+            nansecSinceJ2000 = CDFTT2000.toParse(dateTime);
           } catch (CDFException ex) {
             nansecSinceJ2000 = 0;
           }
