@@ -141,11 +141,11 @@
 
 #define TIME_STAMP_LEN		31
 
-#define EPOCH0_STYLE		0		/* Standard */
+#define EPOCH0_STYLE		0
 #define EPOCH1_STYLE		1
 #define EPOCH2_STYLE		2
 #define EPOCH3_STYLE		3
-#define EPOCH4_STYLE		6
+#define EPOCH4_STYLE		6		/* ISO 8601 -- new standard */
 #define EPOCHf_STYLE		4		/* Use C/Fortran format. */
 #define EPOCHx_STYLE		5		/* Use custom format. */
 
@@ -153,10 +153,11 @@
 * TT2000/time constants.
 *****************************************************************************/
 
-#define TT2000_0_STYLE		0               /* Standard */
+#define TT2000_0_STYLE		0
 #define TT2000_1_STYLE		1
 #define TT2000_2_STYLE		2
-#define TT2000_3_STYLE		3
+#define TT2000_3_STYLE		3		/* ISO 8601 -- Standard */
+#define TT2000_4_STYLE		4		/* ISO 8601 -- Standard */
 
 /*****************************************************************************
 * QOP constants.
@@ -337,7 +338,7 @@ struct HyperStruct {
 #define MAX_ENCODED_INT4_LEN		11
 #define MAX_ENCODED_INT8_LEN		20
 #define MAX_REPORTMASK_LEN		40
-#define MAX_oTEXT_LEN			150
+#define MAX_oTEXT_LEN			1024
 #define MAX_CACHESIZES_LEN		30
 #define MAX_TOLERANCES_LEN              30
 
@@ -646,7 +647,7 @@ extern int lastSecond;
 * Function prototypes.
 *****************************************************************************/
 
-int UTF8StrLength PROTOARGs((unsigned char *));
+int UTF8StrLength PROTOARGs((unsigned char *, int));
 void ASSIGNx PROTOARGs((void *, void *, long, long));
 Logical EQx PROTOARGs((void *, void *, long, long));
 Logical NEx PROTOARGs((void *, void *, long, long));
@@ -706,7 +707,7 @@ int EncodeValuesFormat PROTOARGs((
 ));
 void EncodeNegativeZero PROTOARGs((char *string, char *format, size_t width));
 void WriteStringValue PROTOARGs((FILE *, long, void *, int, int));
-void WriteEntryValue PROTOARGs((FILE *, long, long, void *, int, int));
+void WriteEntryValue PROTOARGs((FILE *, long, long, void *, int, int, int));
 char *EncodingToken PROTOARGs((long));
 char *MajorityToken PROTOARGs((long));
 char *FormatToken PROTOARGs((long));
@@ -776,6 +777,7 @@ Logical S6qualifierLong PROTOARGs((
   long defaultS, char *conflictText
 ));
 void BlankPadRight PROTOARGs((char *string, int count));
+void NulPadRight PROTOARGs((char *string, int count));
 CDFstatus GetFormatEntry PROTOARGs((Logical, long, char **));
 Logical ParseOptionList PROTOARGs((
   int nTokens, char *tokens[], char *list, Logical present[]
@@ -792,6 +794,7 @@ int DirList PROTOARGs((
 int CDFdirList PROTOARGs((char *path, char ***dirS, char ***nameS));
 void RemoveExtensions PROTOARGs((int, char **));
 char *TempCharStr PROTOARGs((int character));
+char *EpochDateTimeString PROTOARGs((long datatype, int style, double *epoch));
 void DisplayStatistics PROTOARGs((
   char *label, vSTATS *vStatsDotCDF, vSTATS *vStatsStage,
   vSTATS *vStatsCompress
