@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 1996-2013 United States Government as represented by the
+* Copyright 1996-2014 United States Government as represented by the
 * Administrator of the National Aeronautics and Space Administration.
 * All Rights Reserved.
 ******************************************************************************/
@@ -475,8 +475,8 @@ switch (Va->item) {
     int entryType = E3p(Va->item,gENTRY_EXISTENCE_,
 				 rENTRY_EXISTENCE_,
 				 zENTRY_EXISTENCE_);
-    struct CDFstruct *CDF; Logical zEntry;
-    long entryNum = va_arg (Va->ap, long), entryN;
+    struct CDFstruct *CDF; Logical zEntry = FALSE;
+    long entryNum = va_arg (Va->ap, long), entryN = -1L;
     SelectCDF (Cur->cdf, CDF)
     if (!CURRENTattrSELECTED(CDF)) return NO_ATTR_SELECTED;
     if (!sX(CheckEntryOp(CDF,entryType),&pStatus)) return pStatus;
