@@ -774,7 +774,7 @@ switch (Va->item) {
       }
       if (vType == SPARSE_RECORDS_) {
           if (Var != NULL) {
-            if (Var->stage.areaOffset64 != (OFF_T) NO_OFFSET64 &&
+            if (Var->stage.areaOffset != (Int32) NO_OFFSET &&
                 Var->stage.firstRec != NO_RECORD) {
               Int32 tmp = (Var->stage.lastRec > maxRec ? maxRec :
                            Var->stage.lastRec);
@@ -784,9 +784,9 @@ switch (Va->item) {
       } else if (vType == COMPRESSED_ ||
                  vType == SPARSE_COMPRESSED_RECORDS_) {
         if (Var != NULL) {
-          if (Var->stage.areaOffset64 != (OFF_T) NO_OFFSET64 &&
+          if (Var->stage.areaOffset != (Int32) NO_OFFSET &&
               Var->stage.firstRec != NO_RECORD &&
-              Var->stage.dotOffset64 == (OFF_T) NO_OFFSET64) {
+              Var->stage.dotOffset == (Int32) NO_OFFSET) {
             Int32 tmp = (Var->stage.lastRec > maxRec ? maxRec :
                          Var->stage.lastRec);
             lastAllocatedRecN += (tmp - Var->stage.firstRec + 1);
@@ -1351,7 +1351,7 @@ switch (Va->item) {
         of the pad value is greater than 16, it causes segmentation fault
         for the IDL CDF interface routine.
     ***********************************************************************/
-    if (dataType == CDF_CHAR ||  dataType == CDF_UCHAR) 
+    if (STRINGdataType(dataType)) 
         numElems = 1;
 
     if (PADvalueBITset(flags)) {
@@ -1804,6 +1804,74 @@ switch (Va->item) {
     }
     break;
   }
+  /****************************************************************************
+   * rENTRY_NUMSTRINGS_/zENTRY_NUMSTRINGS_
+   ****************************************************************************/
+  case rENTRY_NUMSTRINGS_:
+  case zENTRY_NUMSTRINGS_: {
+    Logical dataOp = (Va->item == rENTRY_NUMSTRINGS_);
+    struct CDFstruct *CDF;
+    long *value = va_arg (Va->ap, long *);
+    Int32 tValue;
+    Int32 eOffset;
+    SelectCDF (Cur->cdf, CDF)
+    if (!CURRENTattrSELECTED(CDF)) return NO_ATTR_SELECTED;
+    if (BOO(dataOp, CDF->CURgrEntryNum, CDF->CURzEntryNum) ==
+        RESERVED_ENTRYNUM) return NO_ENTRY_SELECTED;
+    if (!sX(CheckEntryOp(CDF,Va->item),&pStatus)) return pStatus;
+    eOffset = BOO(dataOp, CDF->CURgrEntryOffset,
+                          CDF->CURzEntryOffset);
+    if (eOffset == (Int32) RESERVED_ENTRYOFFSET) return NO_SUCH_ENTRY;
+    if (!sX(ReadAEDR(CDF->fp,eOffset,
+                     AEDR_NUMSTRINGS,&tValue,
+                     AEDR_NULL),&pStatus)) {
+      AbortAccess (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    *value = (long) BOO((tValue>0), tValue, 0);
+    break;
+  }
+  /****************************************************************************
+   * rENTRY_STRINGSDATA_/zENTRY_STRINGSDATA_, 
+   ****************************************************************************/
+  case rENTRY_STRINGSDATA_:
+  case zENTRY_STRINGSDATA_: {
+    Logical zOp = (Va->item == zENTRY_STRINGSDATA_);
+    struct CDFstruct *CDF;
+    Int32 dataType, numElems;
+    Int32 offset;
+    long *numStrings = va_arg (Va->ap, long *);
+    char ***strings = va_arg (Va->ap, char ***);
+    char *tmp;
+    SelectCDF (Cur->cdf, CDF)
+    if (!CURRENTattrSELECTED(CDF)) return NO_ATTR_SELECTED;
+    if ((!zOp?CDF->CURgrEntryNum:CDF->CURzEntryNum) == RESERVED_ENTRYNUM)
+      return NO_ENTRY_SELECTED;
+    offset = (!zOp?CDF->CURgrEntryOffset:CDF->CURzEntryOffset);
+    if (offset == (Int32) RESERVED_ENTRYOFFSET) return NO_SUCH_ENTRY;
+    if (!sX(ReadAEDR(CDF->fp,offset,
+                     AEDR_DATATYPE,&dataType,
+                     AEDR_NUMELEMS,&numElems,
+                     AEDR_NULL),&pStatus)) {
+      AbortAccess (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    if (!STRINGdataType(dataType)) return TRY_TO_READ_NONSTRING_DATA;
+    tmp = (char *) malloc((size_t)numElems+1);
+    if (!sX(ReadAEDR(CDF->fp,offset,
+                     AEDR_NUMSTRINGS,numStrings,
+                     AEDR_VALUE,tmp,
+                     AEDR_NULL),&pStatus)) {
+      AbortAccess (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    *(tmp+numElems) = '\0';
+    /* Pre-3.7.0, the field was used and filled with 0. */
+    if (*numStrings < 1) *numStrings = 1;
+    StringBreakIntoStrings (tmp, numElems, *numStrings, strings);
+    free (tmp);
+    break;
+  }
   /****************************************************************************
   * Unknown item, must be the next function.
   ****************************************************************************/
