@@ -47,12 +47,12 @@ void *buffer;
   Int32 recNum, nRecords, phyRecN, i; Byte1 *tBuffer, *phyBuffer;
   CDFstatus pStatus = CDF_OK; OFF_T nValues;
   int dimN, dimNt;
-  int firstDim;                 /* Based on majority, the "highest" dimension
+  int firstDim = -1;            /* Based on majority, the "highest" dimension
 				   (changes the slowest in memory/on disk). */
-  int doneDim;                  /* When cycling through the dimensions, the
+  int doneDim = -1;             /* When cycling through the dimensions, the
 				   dimension at which to stop (the dimension
 				   beyond the last). */
-  int dimIncr;                  /* How to cycle through the dimensions.  Incr-
+  int dimIncr = 0;              /* How to cycle through the dimensions.  Incr-
 				   ementing (+1) for row-major, decrementing
 				   (-1) for column-major. */
   /****************************************************************************
@@ -386,12 +386,12 @@ void *buffer;
   Logical fullPhyRec, fullPhyDim[CDF_MAX_DIMS];
   Int32 firstPhyRec, phyRecN; Byte1 *tBuffer; int dimN; Int32 i;
   CDFstatus pStatus = CDF_OK; Byte1 *phyBuffer; OFF_T nValues;
-  int firstDim;                 /* Based on majority, the "highest" dimension
+  int firstDim = -1;            /* Based on majority, the "highest" dimension
 				   (changes the slowest in memory/on disk). */
-  int doneDim;                  /* When cycling through the dimensions, the
+  int doneDim = -1;             /* When cycling through the dimensions, the
 				   dimension at which to stop (the dimension
 				   beyond the last). */
-  int dimIncr;                  /* How to cycle through the dimensions.  Incr-
+  int dimIncr = 0;              /* How to cycle through the dimensions.  Incr-
 				   ementing (+1) for row-major, decrementing
 				   (-1) for column-major. */
   /****************************************************************************
