@@ -63,6 +63,8 @@ switch (Va->item) {
     Logical upper_case_ext, version_numbers, no_append;
     Int32 magicNumber1, magicNumber2, CDRflags, version, release, increment;
     Logical largeFile;
+    int  validate;
+
     /**************************************************************************
     * Get arguments for this operation/item.
     **************************************************************************/
@@ -356,7 +358,8 @@ switch (Va->item) {
      * Validate the CDF to make sure that certain data fields are normal by
      * doing some sanity checks.
      ***********************************************************************/
-    if (CDFgetValidate()) {
+    validate = CDFgetValidate();
+    if (validate) {
       Logical debug;
       if (CDFgetValidateDebug())
         debug = TRUE;
@@ -508,7 +511,7 @@ switch (Va->item) {
     **************************************************************************/
     Cur->cdf = CDF;
     *id = CDF;
-    if (CDF->singleFile && CDF->checksum != NO_CHECKSUM) {
+    if (validate && CDF->singleFile && CDF->checksum != NO_CHECKSUM) {
       if (!sX(CDFconfirmChecksum(*id),&pStatus)) {
         if (!largeFile) 
           AbortAccess (CDF, noUPDATE, noDELETE);
@@ -520,10 +523,10 @@ switch (Va->item) {
         return pStatus;
       }
     }
-    if (CDFgetValidate() && (!PriorTo ("3.5.0", version, release, increment))) {
+    if (validate && (!PriorTo ("3.5.0", version, release, increment))) {
         if (!sX(ValidateCDFTT2000 (*id, (int) CDF->NzVars, (int) CDF->NrVars),
                 &pStatus)) {
-          AbortAccess (CDF, noUPDATE, noDELETE);
+          AbortAccess64 (CDF, noUPDATE, noDELETE);
           cdf_FreeMemory (CDF, NULL);
           Cur->cdf = NULL;
           *id = (CDFid) NULL;
@@ -531,7 +534,7 @@ switch (Va->item) {
         }
     }
     if (!PriorTo ("3.5.0", version, release, increment)) {
-      int fileStatus = CDFgetLeapSecondsTableStatus();
+      CDFgetLeapSecondsTableStatus();
       AddOpenCDFsCount();
     }
     break;
@@ -571,7 +574,7 @@ static CDFstatus ValidateCDFTT2000 (CDFid id, int zVars, int rVars)
   CDFgetLastDateinLeapSecondsTable (&year, &month, &day);
 #endif
   if (tt2000LastUpdated == 0 ||
-      tt2000LastUpdated == (10000*year+100*month+day)) return CDF_OK;
+      tt2000LastUpdated <= (10000*year+100*month+day)) return CDF_OK;
   once = 0;
   for (i = 0; i < zVars; ++i) {
     status = CDFlib(SELECT_, zVAR_, (long) i,
