@@ -270,11 +270,6 @@ size_t max;
     j = MINIMUM(strlen(src), max);
     strncpy (dst, src, j);
     dst[(int)j] = NUL;
-/*
-    int i;
-    for (i = 0; src[i] != NUL && i < (int) max; i++) dst[i] = src[i];
-    dst[i] = NUL;
-*/
   } else if (max == 0) {
     strcpy (dst, src);
   } else if (src[0] == NUL) {
@@ -312,13 +307,6 @@ size_t max;
       if (k > 0) 
         strncat (dst, src, k);
     }
-/*
-    for (i = strlen(dst), j = 0; src[j] != NUL && i < (int) max; i++, j++) {
-       dst[i] = src[j];
-
-    }
-    dst[i] = NUL; 
-*/
   } else if (max == 0) {
     strcat (dst, src);
   }
@@ -451,13 +439,11 @@ int length;
     ptr = FORTstring;
     lenT = (size_t) length;
   }
-/*memmove (ptr, Cstring, lenT);*/
   strncpy (ptr, Cstring, lenT);
   for (i = strlen(Cstring); i < lenT; i++) ptr[i] = ' ';
   return;
 #else
   int i;
-/*memmove (FORTstring, Cstring, (size_t) length);*/
   strncpy (FORTstring, Cstring, (size_t) length);
   for (i = strlen(Cstring); i < length; i++) ((char *) FORTstring)[i] = ' ';
   return;
