@@ -13,6 +13,8 @@
 *
 *   V1.0  15-Mar-11, M Liu      Initial version (for CDF V3.3.2)
 *   V2.0  15-Feb-12, M Liu      Initial version (for CDF V3.4.0)
+*   V2.1  10-Sep-18, M Liu      Added toEncodeTT2000.
+*
 ******************************************************************************/
 
 #include "cdflib.h"
@@ -93,6 +95,21 @@ static double  LTS [][6]= {
 static const int NDAT = sizeof (LTS) / sizeof (LTS[0]);
 static double **LTD = NULL;
 static long long *NST = NULL;
+/* Pre-computed times from CDF_TT2000_from_UTC_parts for days in LTS table */
+static long long NST2 [] = {
+         0LL, 0LL, 0LL, 0LL, 0LL, 0LL, 0LL, 0LL, 0LL, 0LL,
+         0LL, 0LL, 0LL, 0LL,
+         -883655957816000000LL, -867931156816000000LL, -852033555816000000LL,
+         -820497554816000000LL, -788961553816000000LL, -757425552816000000LL,
+         -725803151816000000LL, -694267150816000000LL, -662731149816000000LL,
+         -631195148816000000LL, -583934347816000000LL, -552398346816000000LL,
+         -520862345816000000LL, -457703944816000000LL, -378734343816000000LL,
+         -315575942816000000LL, -284039941816000000LL, -236779140816000000LL,
+         -205243139816000000LL, -173707138816000000LL, -126273537816000000LL,
+          -79012736816000000LL,  -31579135816000000LL,  189345665184000000LL,
+          284040066184000000LL,  394372867184000000LL,  488980868184000000LL,
+          536500869184000000LL};
+
 static int entryCnt;
 static char *leapTableEnv = NULL;
 
@@ -191,15 +208,25 @@ static void DatefromJulianDay (double julday, long *y, long *m, long *d)
 
 static int ScanUTCstring (char *string) 
 {
-  int len;
+  int len; long tmp;
   len = (int) strlen (string);
   if ((len == TT2000_3_STRING_LEN) ||
-      (len > 20 && string[10] == 'T')) return 3;
-  else if ((len == TT2000_0_STRING_LEN) || 
-           (len > 20 && string[11] == ' ')) return 0; 
-  else if ((len == TT2000_1_STRING_LEN) ||
-           (len > 9 && string[8] == '.')) return 1;
-  else if ((len == TT2000_2_STRING_LEN) || (len > 7))  return 2;
+      ((len >= 19 && len < TT2000_3_STRING_LEN) &&
+       (string[10] == 'T' || string[10] == 't' || string[10] == ' ') &&
+        (string[len-1] != 'Z'))) return 3;
+  else if ((len <= TT2000_0_STRING_LEN) &&
+            string[11] == ' ') return 0;
+  else if ((len == TT2000_4_STRING_LEN) ||
+           ((len >= 19 && len < TT2000_4_STRING_LEN) &&
+            (string[10] == 'T' || string[10] == 't' || string[10] == ' ') &&
+            (string[len-1] == 'Z'))) return 4;
+  else if ((len == TT2000_1_STRING_LEN && string[8] == '.') ||
+           ((len > 9 && len < TT2000_1_STRING_LEN) &&
+            string[8] == '.')) return 1;
+  else if (len == TT2000_2_STRING_LEN &&
+           sscanf (string, "%8ld", &tmp) == 1)  return 2;
+  else if (len == (TT2000_0_STRING_LEN + 1) &&
+           ((string[11] == ' ' && string[len-1] == 'Z'))) return 0;
   return -1;
 }
 
@@ -321,10 +348,19 @@ static void LoadLeapNanoSecondsTable ()
   int ix;
   if (LTD == NULL) LoadLeapSecondsTable ();
   NST = (long long *) cdf_AllocateMemory (entryCnt * sizeof (long long), NULL);
-  for (ix = NERA1; ix < entryCnt; ++ix)
-     NST[ix] = CDF_TT2000_from_UTC_parts(LTD[ix][0], LTD[ix][1],
-                                         LTD[ix][2], 0.0, 0.0, 0.0,
-                                         0.0, 0.0, 0.0);
+  if (fromFile == 0)
+    memcpy (NST, NST2, entryCnt * sizeof (long long));
+  else {
+    if (LTD[entryCnt-1][0] == LTS[entryCnt-1][0]) 
+      memcpy (NST, NST2, entryCnt * sizeof (long long));
+    else {
+      for (ix = NERA1; ix < entryCnt; ++ix) {
+         NST[ix] = CDF_TT2000_from_UTC_parts(LTD[ix][0], LTD[ix][1],
+                                             LTD[ix][2], 0.0, 0.0, 0.0,
+                                             0.0, 0.0, 0.0);
+      }
+    }
+  }
 }
 
 static double LeapSecondsfromYMD (long iy, long im, long id)
@@ -433,7 +469,6 @@ VISIBLE_PREFIX void CDFClearLeapSecondsTable ()
     free (leapTableEnv);
     leapTableEnv = NULL;
   }
-/*  tableChecked  = 0; */
 }
 
 /******************************************************************************
@@ -496,17 +531,15 @@ VISIBLE_PREFIX int CDFgetLeapSecondsTableStatus ()
 }
       
 /******************************************************************************
-* CDF_TT2000_to_UTC_parts.
+* breakdownTT2000 (aka CDF_TT2000_to_UTC_parts or TT2000breakdown).
 ******************************************************************************/
 
 #if defined(STDARG)
-VISIBLE_PREFIX void CDF_TT2000_to_UTC_parts (long long nanoSecSinceJ2000,
-                                             double *ly, double *lm,
-                                             double *ld, ...)
+VISIBLE_PREFIX void breakdownTT2000 (long long nanoSecSinceJ2000,
+                                     double *ly, double *lm,
+                                     double *ld, ...)
 #else
-VISIBLE_PREFIX void CDF_TT2000_to_UTC_parts (long long nanoSecSinceJ2000,
-                                             double *ly, double *lm,
-                                             double *ld, va_alist)
+VISIBLE_PREFIX void breakdownTT2000 (va_alist)
 va_dcl
 #endif
 {
@@ -516,21 +549,26 @@ va_dcl
   double *opt[6];
   long ye1, mo1, da1, ho1, mi1, se1, ml1, ma1, na1;
   int  ix, leapSec;
-  va_list Va;
+  va_list ap;
 #if defined(STDARG)
-  va_start (Va, ld);
+  va_start (ap, ld);
 #else
-  VA_START (Va);
+  long long nanoSecSinceJ2000; double *ly, *lm, *ld;
+  VA_START (ap);
+  nanoSecSinceJ2000 = va_arg (ap, long long);
+  ly = va_arg (ap, double *);
+  lm = va_arg (ap, double *);
+  ld = va_arg (ap, double *);
 #endif
   ix = 0;
-  tmp = va_arg (Va, double *);
+  tmp = va_arg (ap, double *);
   while (tmp != TT2000NULL) {
     opt[ix] = tmp;
     ++ix;
     if (ix == 6) break;
-    tmp = va_arg (Va, double *);
+    tmp = va_arg (ap, double *);
   }
-  va_end (Va);
+  va_end (ap);
   if (nanoSecSinceJ2000 == FILLED_TT2000_VALUE) {
     *ly = 9999.0;
     *lm = 12.0;
@@ -705,6 +743,7 @@ VISIBLE_PREFIX long long computeTT2000withBasedLeapDay (long yy, long mm,
   if (yy < 0 || mm < 0 || dd < 0 || hh < 0 || mn < 0 ||
       ss < 0 || ms < 0 || us < 0 || ns < 0)
     return ILLEGAL_TT2000_VALUE;
+  if (mm == 0) mm = 1;
   nanoSecSinceJ2000 = computeTT2000 ((double)yy, (double)mm, (double)dd,
                                      (double)hh, (double)mn, (double)ss,
                                      (double)ms, (double)us, (double)ns);
@@ -805,13 +844,13 @@ VISIBLE_PREFIX void encodeTT2000withBasedLeapDay (long long tt2000,
   long long *tmpTT2000s;
   int off, ix, iy, found;
   if (yymmdd <= 0) {
-    CDF_TT2000_to_UTC_string (tt2000, string);
+    CDF_TT2000_to_UTC_string (tt2000, string, 3);
     return;
   }
   ix = LeapSecondLastUpdatedinTable (yymmdd);
   off = entryCnt - ix - 1;
   if (off == 0) {
-    CDF_TT2000_to_UTC_string (tt2000, string);
+    CDF_TT2000_to_UTC_string (tt2000, string, 3);
     return;
   } else {
     found = 0;
@@ -824,13 +863,14 @@ VISIBLE_PREFIX void encodeTT2000withBasedLeapDay (long long tt2000,
                                                       0.0, 0.0, yymmdd);
     for (iy = off; iy > 0; --iy) {
       if (tt2000 >= tmpTT2000s[iy-1]) {
-        CDF_TT2000_to_UTC_string (tt2000+(long long)iy*1000000000LL, string);
+        CDF_TT2000_to_UTC_string (tt2000+(long long)iy*1000000000LL, string,
+                                  3);
         found = 1;
         break;
       }
     }
     if (found == 0)
-      CDF_TT2000_to_UTC_string (tt2000, string);
+      CDF_TT2000_to_UTC_string (tt2000, string, 3);
     free (tmpTT2000s);
   }
   return;
@@ -860,15 +900,14 @@ VISIBLE_PREFIX long long parseTT2000withBasedLeapDay (char *tt2000, int yymmdd)
 }
 
 /******************************************************************************
-* CDF_TT2000_from_UTC_parts.
+* computeTT2000 (aka CDF_TT2000_from_UTC_parts).
 ******************************************************************************/
 
 #if defined(STDARG)
-VISIBLE_PREFIX long long CDF_TT2000_from_UTC_parts (double yy, double mm, 
-                                                    double dd, ...)
+VISIBLE_PREFIX long long computeTT2000 (double yy, double mm, 
+                                        double dd, ...)
 #else
-VISIBLE_PREFIX long long CDF_TT2000_from_UTC_parts (double yy, double mm,
-                                                    double dd, va_alist)
+VISIBLE_PREFIX long long computeTT2000 (va_alist)
 va_dcl
 #endif
 {
@@ -876,27 +915,32 @@ va_dcl
   long long subDayinNanoSecs, nanoSecSinceJ2000;
   long long t2, iy;
   double tmp, opt[6];
-  va_list Va;
-  int ix;
+  va_list ap;
+  int  ix;
   double ly, lm, ld, lh, ln, ls, ll, lu, la;
   long lyl, lml, ldl, lhl, lnl, lsl, lll, lul, lal;
   long xy, xm, xd;
-  int lyear;
+  int  lyear;
 
 #if defined(STDARG)
-  va_start (Va, dd);
+  va_start (ap, dd);
 #else
-  VA_START (Va);
+  double yy, mm, dd;
+  VA_START (ap);
+  yy = va_arg (ap, double);
+  mm = va_arg (ap, double);
+  dd = va_arg (ap, double);
 #endif
   ix = 0;
-  tmp = va_arg (Va, double);
+  tmp = va_arg (ap, double);
   while (tmp != TT2000END) {
     opt[ix] = tmp;
     ++ix;
     if (ix == 6) break;
-    tmp = va_arg (Va, double);
+    tmp = va_arg (ap, double);
   }
-  va_end (Va);
+  va_end (ap);
+  if (mm == 0.0) mm = 1.0;
   ly = floor(yy);
   lm = floor(mm);
   ld = floor(dd);
@@ -1114,7 +1158,8 @@ va_dcl
   lul = (long) lu;
   lal = (long) la;
   if (lyl == 9999 && lml == 12 && ldl == 31 && lhl == 23 && lnl == 59 &&
-      lsl == 59 && lll == 999 && lul == 999 && lal == 999)
+/*      lsl == 59 && lll == 999 && lul == 999 && lal == 999) */
+      lsl == 59 && lll == 999)
     return FILLED_TT2000_VALUE;
   else if (lyl == 0 && lml == 1 && ldl == 1 && lhl == 0 && lnl == 0 &&
            lsl == 0 && lll == 0 && lul == 0 && lal == 0)
@@ -1250,58 +1295,91 @@ VISIBLE_PREFIX long long CDF_TT2000_from_UTC_EPOCH16 (double *epoch)
 }
 
 /******************************************************************************
-* CDF_TT2000_to_UTC_string.
+* toEncodeTT2000 - to encode a TT2000 data into readable date/time string based
+* on the style.
+* Style 0: dd-mon-yyyy hh:mm:ss.mmmuuunnn as 31-Dec-9999 23:59:59.999999999
+* Style 1: yyyymmdd.tttttttttt as 99991231.9999999999
+* Style 2: yyyymmddhhmnss as 99991231235959
+* Style 3: yyyy-mm-ddThh:mn:ss.mmmuuunnn as 9999-12-31T23:59:59.999999999
+* Style 4: yyyy-mm-ddThh:mn:ss.mmmuuunnnZ as 9999-12-31T23:59:59.999999999Z
+******************************************************************************/
+
+VISIBLE_PREFIX void toEncodeTT2000 (long long nanoSecSinceJ2000,
+                                    int style, char *string)
+{
+   if (style < 0 || style > 4) style = 3;
+   encodeTT2000 (nanoSecSinceJ2000, string, style);
+}
+
+/******************************************************************************
+* encodeTT2000 (aka CDF_TT2000_to_UTC_string).
 ******************************************************************************/
 
 #if defined(STDARG)
+/*
 VISIBLE_PREFIX void CDF_TT2000_to_UTC_string (long long nanoSecSinceJ2000,
                                               char *string, ...)
+*/
+VISIBLE_PREFIX void encodeTT2000 (long long nanoSecSinceJ2000,
+                                  char *string, ...)
 #else
-VISIBLE_PREFIX void CDF_TT2000_to_UTC_string (long long nanoSecSinceJ2000,
-                                              char *string, va_alist)
+/*
+VISIBLE_PREFIX void CDF_TT2000_to_UTC_string (va_alist)
+*/
+VISIBLE_PREFIX void encodeTT2000 (va_alist)
 va_dcl
 #endif
 {
   long ly, lm, ld, lh, ln, ls, ll, lu, la;
   double lyd, lmd, ldd, lhd, lnd, lsd, lld, lud, lad;
-  int format;
-  va_list Va;
+  int style;
+  va_list ap;
 #if defined(STDARG)
-  va_start (Va, string);
+  va_start (ap, string);
 #else
-  VA_START (Va);
+  long long nanoSecSinceJ2000; char *string;
+  VA_START (ap);
+  nanoSecSinceJ2000 = va_arg (ap, long long);
+  string = va_arg (ap, char *);
 #endif
-  format = va_arg (Va, int);
-  if (format < 0 || format > 3) format = 3;
-  va_end (Va);
-  if (nanoSecSinceJ2000 == FILLED_TT2000_VALUE) {
-    if (format == 0) {
-      strncpy(string, "31-DEC-9999 23:59:59.999999999", 30);
+  style = va_arg (ap, int);
+  if (style < 0 || style > 4) style = 3;
+  va_end (ap);
+  if (nanoSecSinceJ2000 == FILLED_TT2000_VALUE ||
+      nanoSecSinceJ2000 == ILLEGAL_TT2000_VALUE) {
+    if (style == 0) {
+      strncpy(string, "31-Dec-9999 23:59:59.999999999", 30);
       string[30] = '\0';
-    } else if (format == 1) {
+    } else if (style == 1) {
       strncpy(string, "99991231.9999999999", 19);
       string[19] = '\0';
-    } else if (format == 2) {
+    } else if (style == 2) {
       strncpy(string, "99991231235959", 14);
       string[14] = '\0';
-    } else {
+    } else if (style == 3) {
       strncpy(string, "9999-12-31T23:59:59.999999999", 29);
       string[29] = '\0';
+    } else {
+      strncpy(string, "9999-12-31T23:59:59.999999999Z", 30);
+      string[30] = '\0';
     }
     return;
   } else if (nanoSecSinceJ2000 == DEFAULT_TT2000_PADVALUE) {
-    if (format == 0) {
+    if (style == 0) {
       strncpy(string, "01-JAN-0000 00:00:00.000000000", 30);
       string[30] = '\0';
-    } else if (format == 1) {
+    } else if (style == 1) {
       strncpy(string, "00000101.0000000000", 19);
       string[19] = '\0';
-    } else if (format == 2) {
+    } else if (style == 2) {
       strncpy(string, "00000101000000", 14);
       string[14] = '\0';
-    } else {
+    } else if (style == 3) {
       strncpy(string, "0000-01-01T00:00:00.000000000", 29);
       string[29] = '\0';
+    } else {
+      strncpy(string, "0000-01-01T00:00:00.000000000Z", 30);
+      string[30] = '\0';
     }
     return;
   }
@@ -1316,11 +1394,11 @@ va_dcl
   ll = (long) lld;
   lu = (long) lud;
   la = (long) lad;
-  if (format == 0) {
+  if (style == 0) {
     snprintf(string, (size_t) TT2000_0_STRING_LEN+1, 
              "%2.2ld-%s-%4.4ld %2.2ld:%2.2ld:%2.2ld.%3.3ld%3.3ld%3.3ld",
              ld,MonthToken(lm),ly,lh,ln,ls,ll,lu,la);
-  } else if (format == 1) {
+  } else if (style == 1) {
     long milsecs, nansecs;
     double subday;
     long long subdayll;
@@ -1331,46 +1409,119 @@ va_dcl
     subdayll = (long long) (subday * pow (10.0, 10));
     snprintf(string, (size_t) TT2000_1_STRING_LEN+1,
              "%4.4ld%2.2ld%2.2ld.%10.10lld", ly,lm,ld,subdayll);
-  } else if (format == 2) {
+  } else if (style == 2) {
     snprintf(string, (size_t) TT2000_2_STRING_LEN+1,
              "%4.4ld%2.2ld%2.2ld%2.2ld%2.2ld%2.2ld",ly,lm,ld,lh,ln,ls);
-  } else if (format == 3) {
-    snprintf(string, (size_t) TT2000_3_STRING_LEN+1,
-             "%4.4ld-%2.2ld-%2.2ldT%2.2ld:%2.2ld:%2.2ld.%3.3ld%3.3ld%3.3ld",
-             ly,lm,ld,lh,ln,ls,ll,lu,la);
+  } else if (style == 3 || style == 4) {
+    snprintf(string, (size_t) (style==3?(TT2000_3_STRING_LEN+1):
+                                        (TT2000_4_STRING_LEN+1)),
+             "%4.4ld-%2.2ld-%2.2ldT%2.2ld:%2.2ld:%2.2ld.%3.3ld%3.3ld%3.3ld%s",
+             ly,lm,ld,lh,ln,ls,ll,lu,la,(style==4?"Z":""));
   }
 }
 
 /******************************************************************************
-* CDF_TT2000_from_UTC_string.
+* toParseTT2000
+* This function parses an input date/time string and returns an TT2000
+* value.  The string format must be exactly as one of the followings shown
+* below:
+* 
+* Style: 0       dd-mmm-yyyy hh:mm:ss.mmmuuunnn (len = 30)
+* Examples:       1-Apr-1990 03:05:02.000111222
+*                10-Oct-1993 23:45:49.999999999
+* 
+* Style: 1       yyyymmdd.tttttttttt (len = 19)
+* Examples:      19950508.0000000000
+*                19671231.58      (== 19671213.5800000000)
+* 
+* Style: 2       yyyymmddhhmmss (len = 14)
+* Examples:      19950508000000
+*                19671231235959
+* 
+* Style: 3       yyyy-mm-ddThh:mm:ss.cccuuunnn (len = 29)
+* Examples:      1990-04-01T03:05:02.000111222
+*                1993-10-10T23:45:49.999999999
+* 
+* Style: 4       yyyy-mm-ddThh:mm:ss.cccuuunnnZ (len = 30)
+* Examples:      1990-04-01T03:05:02.000111222Z
+*                1993-10-10T23:45:49.999999999Z
+*
 ******************************************************************************/
 
-VISIBLE_PREFIX long long CDF_TT2000_from_UTC_string (char *string)
+VISIBLE_PREFIX long long toParseTT2000 (char *string)
+{
+  return parseTT2000(string);
+}
+
+
+/******************************************************************************
+* parseTT2000 (aka CDF_TT2000_from_UTC_string).
+******************************************************************************/
+
+VISIBLE_PREFIX long long parseTT2000 (char *string)
 {
   double fraction;
   long ly, lm, ld, lh, ln, ls, ll, lu, la;
-  int len, format;
+  int  len, style, is, ie;
+  char *tmpPtr, *end;
 
-  while (string[0] == ' ') memmove (string, &string[1], strlen(string));
-  format = ScanUTCstring(string);
-  if (format == 0) {
+  ll = lu = la = 0L;
+  is = ie = 0;
+  len = (int) strlen(string);
+  while(isspace((unsigned char)*(string+is)) || 
+        !isprint((unsigned char)*(string+is))) is++;
+  while(isspace((unsigned char)*(string+len-ie-1)) ||
+        !isprint((unsigned char)*(string+len-ie-1))) ie++;
+  len = len - is - ie;
+  tmpPtr = (char *) malloc ((size_t)len+1);
+  memcpy (tmpPtr, string+is, (size_t)len);
+  *(tmpPtr+len) = (char) '\0';
+  style = ScanUTCstring(tmpPtr);
+  if (style == 0) {
     char moString[4];
     long tmp;
-    if (sscanf(string,"%2ld-%c%c%c-%4ld %2ld:%2ld:%2ld.%9ld",
-        &ld, &(moString[0]), &(moString[1]), &(moString[2]), &ly,
-        &lh, &ln, &ls, &tmp) != 9) return ILLEGAL_TT2000_VALUE;
+    char *newstr = NULL;
+    len = (int) strlen(tmpPtr);
+    if (len != TT2000_0_STRING_LEN) {
+      int ix;
+      newstr = (char *) malloc (TT2000_0_STRING_LEN+1);
+      for (ix = 0; ix < TT2000_0_STRING_LEN+1; ++ix) newstr[ix] = '0';
+      newstr[TT2000_0_STRING_LEN] = '\0';
+      if (tmpPtr[len-1] == 'Z' || tmpPtr[len-1] == 'z') --len;
+      memcpy (newstr, tmpPtr, len);
+      tmp = 0L;
+      if (sscanf(newstr,"%2ld-%c%c%c-%4ld %2ld:%2ld:%2ld.%9ld",
+          &ld, &(moString[0]), &(moString[1]), &(moString[2]), &ly,
+          &lh, &ln, &ls, &tmp) < 8) {
+         free (newstr);
+         free (tmpPtr);
+         return ILLEGAL_TT2000_VALUE;
+      }
+    } else {
+      if (sscanf(tmpPtr,"%2ld-%c%c%c-%4ld %2ld:%2ld:%2ld.%9ld",
+          &ld, &(moString[0]), &(moString[1]), &(moString[2]), &ly,
+          &lh, &ln, &ls, &tmp) != 9) {
+        free (tmpPtr);
+        return ILLEGAL_TT2000_VALUE;
+      }
+    }
     moString[3] = '\0';
     lm = (long) MonthNumber(moString);
     if (ly == 9999 && lm == 12 && ld == 31 && lh == 23 && ln == 59 &&
-        ls == 59 && tmp == 999999999)
+        ls == 59 && (tmp == 999999999 || tmp == 999 || tmp == 999999)) {
+      free (tmpPtr);
       return FILLED_TT2000_VALUE;
-    else if (ly == 0 && lm == 1 && ld == 1 && lh == 0 && ln == 0 &&
-             ls == 0 && tmp == 0)
+    } else if (ly == 0 && lm == 1 && ld == 1 && lh == 0 && ln == 0 &&
+               ls == 0 && tmp == 0) {
+      free (tmpPtr);
       return DEFAULT_TT2000_PADVALUE;
+    }
     if (tmp == 0) {
       ll = lu = la = 0;
     } else {
-      char *dot = strrchr (string, '.');
+      char *dot;
+      if (newstr == NULL) dot = strrchr (tmpPtr, '.');
+      else dot = strrchr (newstr, '.');
       if (dot == NULL) {
         ll = lu = la = 0;
       } else {
@@ -1382,26 +1533,37 @@ VISIBLE_PREFIX long long CDF_TT2000_from_UTC_string (char *string)
         la = (long) (tmp - lu * 1000);
       }
     }
+    if (newstr != NULL) free (newstr);
     if (!YearWithin(ly) && !ValidateYMD(ly,lm,ld)) return ILLEGAL_TT2000_VALUE;
+    free (tmpPtr);
     return CDF_TT2000_from_UTC_parts ((double) ly, (double) lm, (double) ld,
                                       (double) lh, (double) ln, (double) ls,
                                       (double) ll, (double) lu, (double) la);
-  } else if (format == 1) {
+  } else if (style == 1) {
     long long tmp;
-    if (sscanf(string,"%4ld%2ld%2ld.%lld",
-        &ly, &lm, &ld, &tmp) != 4) return ILLEGAL_TT2000_VALUE;
+    if (sscanf(tmpPtr,"%4ld%2ld%2ld.%lld",
+        &ly, &lm, &ld, &tmp) != 4) {
+      free (tmpPtr);
+      return ILLEGAL_TT2000_VALUE;
+    }
     if (tmp == 0) 
       fraction = 0.0;
     else {
-      char *dot = strrchr (string, '.');
+      char *dot = strrchr (tmpPtr, '.');
       len = (int) strlen (dot + 1);
       fraction = tmp / pow(10.0, len);
     }
-    if (ly == 9999 && lm == 12 && ld == 31 && tmp == 9999999999LL)
+    if (ly == 9999 && lm == 12 && ld == 31 && tmp == 9999999999LL) {
+      free (tmpPtr);
       return FILLED_TT2000_VALUE;
-    else if (ly == 0 && lm == 1 && ld == 1 && tmp == 0)
+    } else if (ly == 0 && lm == 1 && ld == 1 && tmp == 0) {
+      free (tmpPtr);
       return DEFAULT_TT2000_PADVALUE;
-    if (!YearWithin(ly) && !ValidateYMD(ly,lm,ld)) return ILLEGAL_TT2000_VALUE;
+    }
+    if (!YearWithin(ly) && !ValidateYMD(ly,lm,ld)) {
+      free (tmpPtr);
+      return ILLEGAL_TT2000_VALUE;
+    }
     fraction *= 24.0;
     lh = (long) floor(fraction);
     fraction = (fraction - lh) * 60.0;
@@ -1413,47 +1575,97 @@ VISIBLE_PREFIX long long CDF_TT2000_from_UTC_string (char *string)
     fraction = (fraction - ll) * 1000.0;
     lu = (long) floor(fraction);
     la = (long) ((fraction - lu) * 1000.0);
+    free (tmpPtr);
     return CDF_TT2000_from_UTC_parts ((double) ly, (double) lm, (double) ld,
                                       (double) lh, (double) ln, (double) ls,
                                       (double) ll, (double) lu, (double) la);
-  } else if (format == 2) {
-    if (sscanf(string,"%4ld%2ld%2ld%2ld%2ld%2ld",
-        &ly, &lm, &ld, &lh, &ln, &ls) != 6) return ILLEGAL_TT2000_VALUE;
+  } else if (style == 2) {
+    if (sscanf(tmpPtr,"%4ld%2ld%2ld%2ld%2ld%2ld",
+        &ly, &lm, &ld, &lh, &ln, &ls) != 6) {
+      free (tmpPtr);
+      return ILLEGAL_TT2000_VALUE;
+    }
     if (ly == 9999 && lm == 12 && ld == 31 && lh == 23 && ln == 59 &&
-        ls == 59) return FILLED_TT2000_VALUE;
-    else if (ly == 0 && lm == 1 && ld == 1 && lh == 0 && ln == 0 &&
-             ls == 0) return DEFAULT_TT2000_PADVALUE;
-    if (!YearWithin(ly) && !ValidateYMD(ly,lm,ld)) return ILLEGAL_TT2000_VALUE;
+        ls == 59) {
+      free (tmpPtr);
+      return FILLED_TT2000_VALUE;
+    } else if (ly == 0 && lm == 1 && ld == 1 && lh == 0 && ln == 0 &&
+               ls == 0) {
+      free (tmpPtr);
+      return DEFAULT_TT2000_PADVALUE;
+    }
+    if (!YearWithin(ly) && !ValidateYMD(ly,lm,ld)) {
+      free (tmpPtr);
+      return ILLEGAL_TT2000_VALUE;
+    }
+    free (tmpPtr);
     return CDF_TT2000_from_UTC_parts ((double) ly, (double) lm, (double) ld,
                                       (double) lh, (double) ln, (double) ls,
                                       0.0, 0.0, 0.0);
-  } else if (format == 3) {
-    long tmp;
-    if (sscanf(string,"%4ld-%2ld-%2ldT%2ld:%2ld:%2ld.%9ld",
-        &ly, &lm, &ld, &lh, &ln, &ls, &tmp) != 7) return ILLEGAL_TT2000_VALUE;
+  } else if (style == 3 || style == 4) {
+    int len; long tmp; char string2[TT2000_0_STRING_LEN+1];
+    char ac, tmp2[10], *dot;
+    strcpy (tmp2, "000000000");
+    len = (int) strlen (tmpPtr);
+    dot = strrchr (tmpPtr, '.');
+    if (dot == NULL) {
+      if (tmpPtr[len-1] != 'Z') {
+        if (sscanf(tmpPtr,"%4ld-%2ld-%2ld%c%2ld:%2ld:%2ld",
+            &ly, &lm, &ld, &ac, &lh, &ln, &ls) != 7) {
+          free (tmpPtr);
+          return ILLEGAL_TT2000_VALUE;
+        }
+      } else {
+        if (sscanf(tmpPtr,"%4ld-%2ld-%2ld%c%2ld:%2ld:%2ldZ",
+            &ly, &lm, &ld, &ac, &lh, &ln, &ls) != 7) {
+          free (tmpPtr);
+          return ILLEGAL_TT2000_VALUE;
+        }
+      }
+    } else {
+      if (sscanf(tmpPtr,"%4ld-%2ld-%2ld%c%2ld:%2ld:%2ld.",
+          &ly, &lm, &ld, &ac, &lh, &ln, &ls) != 7) {
+        free (tmpPtr);
+        return ILLEGAL_TT2000_VALUE;
+      }
+      if (tmpPtr[len-1] != 'Z') { 
+        memcpy (tmp2, tmpPtr+20, len-20);
+        tmp2[9] = '\0';
+      } else {
+        memcpy (tmp2, tmpPtr+20, len-20-1);
+        tmp2[9] = '\0';
+      }
+    }
+    if (sscanf(tmp2,"%ld", &tmp) != 1) return ILLEGAL_TT2000_VALUE;
     if (ly == 9999 && lm == 12 && ld == 31 && lh == 23 && ln == 59 &&
-        ls == 59 && tmp == 999999999)
+        ls == 59 && (tmp == 999999999 || tmp == 999 || tmp == 999999)) {
+      free (tmpPtr);
       return FILLED_TT2000_VALUE;
-    else if (ly == 0 && lm == 1 && ld == 1 && lh == 0 && ln == 0 &&
-             ls == 0 && tmp == 0)
+    } else if (ly == 0 && lm == 1 && ld == 1 && lh == 0 && ln == 0 &&
+               ls == 0 && tmp == 0) {
+      free (tmpPtr);
       return DEFAULT_TT2000_PADVALUE;
+    }
     if (tmp == 0) {
       ll = lu = la = 0;
     } else {
-      char *dot = strrchr (string, '.');
-      len = (int) strlen (dot+1);
-      if (len < 9) tmp = tmp * pow (10.0, 9-len);
       ll = (long) (tmp / 1000000);
       tmp = tmp - ll * 1000000;
       lu = (long) (tmp / 1000);
       la = (long) (tmp - lu * 1000);
     }
-    if (!YearWithin(ly) && !ValidateYMD(ly,lm,ld)) return ILLEGAL_TT2000_VALUE;
+    if (!YearWithin(ly) && !ValidateYMD(ly,lm,ld)) {
+      free (tmpPtr);
+      return ILLEGAL_TT2000_VALUE;
+    }
+    free (tmpPtr);
     return CDF_TT2000_from_UTC_parts ((double) ly, (double) lm, (double) ld,
                                       (double) lh, (double) ln, (double) ls,
                                       (double) ll, (double) lu, (double) la);
-  } else
+  } else {
+    free (tmpPtr);
     return ILLEGAL_TT2000_VALUE;
+  }
 }
 
 /******************************************************************************
@@ -1564,23 +1776,107 @@ static void FromUTCISO8601string (char *string, long *ly, long *lm, long *ld,
                                   long *lh, long *ln, long *ls,
                                   long *ll, long *lu, long *la)
 {
-  int format, len;
-  char *dot;
+  int style;
 
-  format = ScanUTCstring(string);
-  if (format == 3) { /* %4ld-%2ld-%2ldT%2ld:%2ld:%2ld.%9ld */
-    long tmp;
+  style = ScanUTCstring(string);
+  if (style == 3 || style == 4) { 
+    /* %4ld-%2ld-%2ldT%2ld:%2ld:%2ld[.%9ld] */
+    /* or, %4ld-%2ld-%2ldT%2ld:%2ld:%2ld[.%9ld]Z */
+    int len; long tmp;
+    char ac, tmp2[10], *dot;
     *ly = *lm = *ld = *lh = *ln = *ls = *ll = *lu = *la = 0L;
-    if (sscanf(string,"%4ld-%2ld-%2ldT%2ld:%2ld:%2ld.%9ld",
-        ly, lm, ld, lh, ln, ls, &tmp) != 7) return;
+    strcpy (tmp2, "000000000");
+    len = (int) strlen (string);
     dot = strrchr (string, '.');
-    len = (int) strlen (dot+1);
-    if (len < 9) tmp = tmp * (long) pow (10.0, 9-len);
-    *ll = (long) (tmp / 1000000L);
-    tmp = tmp - *ll * 1000000L;
-    *lu = (long) (tmp / 1000L);
-    *la = (long) (tmp - *lu * 1000L);
+    if (dot == NULL) {
+      if (string[len-1] != 'Z') {
+        if (sscanf(string,"%4ld-%2ld-%2ld%c%2ld:%2ld:%2ld",
+            ly, lm, ld, &ac, lh, ln, ls) != 7) return;
+      } else {
+        if (sscanf(string,"%4ld-%2ld-%2ld%c%2ld:%2ld:%2ldZ",
+            ly, lm, ld, &ac, lh, ln, ls) != 7) return;
+      }
+    } else {
+      if (sscanf(string,"%4ld-%2ld-%2ld%c%2ld:%2ld:%2ld.",
+          ly, lm, ld, &ac, lh, ln, ls) != 7) return;
+      if (string[len-1] != 'Z') { 
+        memcpy (tmp2, string+20, len-20);
+      } else {
+        memcpy (tmp2, string+20, len-20-1);
+      }
+    }
+    if (sscanf(tmp2,"%ld", &tmp) != 1) return;
+    if (tmp == 0) {
+      *ll = *lu = *la = 0;
+    } else {
+      *ll = (long) (tmp / 1000000);
+      tmp = tmp - *ll * 1000000;
+      *lu = (long) (tmp / 1000);
+      *la = (long) (tmp - *lu * 1000);
+    }
   }
 }
 
+/******************************************************************************
+ * LastLeapSecondDayInStaticTable.
+ * Returns the last leap second day added in the static leap second table.
+ ******************************************************************************/
+
+int LastLeapSecondDayInStaticTable ()
+{
+  return (int) LASTLEAPSECONDDAY;
+}
+
+/******************************************************************************
+ * TT2000toUnixTime.
+ * Converts an array of CDF's TT2000 times (nanoseconds from 
+ * 2000-01-01 12:00:00.000) in 64-bit integer to unix times (seconds from 
+ * 1970-01-01 00:00:00 UTC) in double.
+ ******************************************************************************/
+
+VISIBLE_PREFIX void TT2000toUnixTime (epoch, unixTime, numTimes)
+long long *epoch;
+double *unixTime;
+int    numTimes;
+{ 
+  int i; double value, yy, mm, dd, hh, mn, ss, ms, us, ns; 
+  for (i = 0; i < numTimes; ++i) {
+    breakdownTT2000 (*(epoch+i), &yy, &mm, &dd, &hh, &mn, &ss, &ms, &us, &ns);
+    value = computeEPOCH ((long) yy, (long) mm, (long) dd, (long) hh,
+                          (long) mn, (long) ss, (long) ms);
+    if (ns > 500.0) us = us + 1.0;
+    *(unixTime+i) = (value - BeginUnixTimeEPOCH) * pow(10.0, -3.0) +
+                     us * pow(10.0, -6.0);
+  }
+}
+
+/******************************************************************************
+ * UnixTimetoTT2000.
+ * Converts an array of unix times (seconds from 1970-01-01 00:00:00 UTC) in
+ * double to CDF's TT2000 times (nanoseconds from 2000-01-01 12:00:00.000) in 
+ * 64-bit long.
+ ******************************************************************************/
+
+VISIBLE_PREFIX void UnixTimetoTT2000 (unixTime, epoch, numTimes)
+double *unixTime;
+long long *epoch;
+int    numTimes;
+{
+  int i; long long tt2000, tmp2;
+  long yy, mm, dd, hh, mn, ss, ms, us;
+  double value, tmp;
+  for (i = 0; i < numTimes; ++i) {
+    tmp =  *(unixTime+i) * pow(10.0, 3.0);
+    tmp2 = (long long) tmp;
+    value =  (double) tmp2 + BeginUnixTimeEPOCH;
+    EPOCHbreakdown (value, &yy, &mm, &dd, &hh, &mn, &ss, &ms);
+    tmp = (tmp - tmp2) * pow(10.0, 3.0);
+    us = (long) tmp;
+    if ((tmp - us) > 0.5) us = us + 1;
+    tt2000 = computeTT2000 ((double) yy, (double) mm, (double) dd, (double) hh,
+                            (double) mn, (double) ss, (double) ms, (double) us,
+                            0.0);
+    *(epoch+i) = tt2000;
+  }
+}
 
