@@ -493,6 +493,12 @@ static CDFstatus ValidateADR (struct CDFstruct *CDF, vFILE *fp, OFF_T offset,
   if (ADR.MAXgrEntry < -1) 
     return QuitCDF ("CDF(ADR): max g/rEntry is invalid ", offset,
                     4, 1, &(ADR.MAXgrEntry), 0, debug);
+  if ((ADR.Scope == GLOBAL_SCOPE) && (ADR.NzEntries > 0))
+    return QuitCDF ("CDF(ADR): global attribute zEntry is invalid ", offset,
+                    4, 1, &(ADR.NzEntries), 0, debug);
+  if ((ADR.Scope == GLOBAL_SCOPE) && (ADR.AzEDRhead > 0))
+    return QuitCDF ("CDF(ADR): global attribute zEntry link is invalid ", offset,
+                    8, 1, &(ADR.AzEDRhead), 0, debug);
   if ((ADR.Scope == VARIABLE_SCOPE) && (ADR.MAXgrEntry < (ADR.NgrEntries-1)))
     return QuitCDF ("CDF(ADR): max rEntry is invalid ", offset,
                     4, 2, &(ADR.MAXgrEntry), &(ADR.NgrEntries), debug);
