@@ -503,7 +503,7 @@ vFILE *vFp;
 #endif
   FILE *fp = NULL;
   int writable;
-#if defined(unix)
+#if defined(unix) || defined(vms)
   int fileid;
 #endif
 #if defined(win32)
@@ -1408,9 +1408,8 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
   * scratch file to which nothing has been written.
   ****************************************************************************/
   if (vFp->fp != NULL) {
-    if (CDF != NULL && (!CDF->readOnly || CDF->status == READ_WRITE) && 
+    if (CDF != NULL && !(CDF->readOnly || CDF->status == READ_ONLY) && 
         CDF->singleFile && (CDF->checksum != NONE_CHECKSUM)) {
-/*      if (!FLUSHv(vFp)) error = TRUE; */
       if (!CDFAddChecksum(CDF)) error = TRUE;
     }
     if (fclose(vFp->fp) == EOF) error = TRUE;
