@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 1996-2013 United States Government as represented by the
+* Copyright 1996-2014 United States Government as represented by the
 * Administrator of the National Aeronautics and Space Administration.
 * All Rights Reserved.
 ******************************************************************************/
@@ -28,6 +28,7 @@
 *   V3.3  10-Apr-09, M Liu      Filled blank(s) for default pad value of
 *                               CDF_CHAR type.
 *   V3.3  04-Apr-11, M Liu      Modified to handle TT2000 epoch style string.
+*   V3.5  27-May-14, M Liu      Broke "Delete records" into two options.
 *
 ******************************************************************************/
 
@@ -40,6 +41,9 @@
 static Logical ModPadValue PROTOARGs((
   Logical zVar, long varN, Logical useFormat, Logical *changed
 ));
+static Logical RegularVar PROTOARGs((
+  Logical zVar, long varN
+));
 
 /******************************************************************************
 * EditVars.  This function handles both rVariables and zVariables.
@@ -554,8 +558,8 @@ struct ItemWindowStruct *IW;
       EncodeString (strlen(varName), varName, field, 0, VARNAME_FIELD_LEN);
       strcpyX (IW->iLines[(int)varN], field, 0);
       if (strlen(field) < (size_t) VARNAME_FIELD_LEN) {
-		CatNcharacters (IW->iLines[(int)varN],
-						VARNAME_FIELD_LEN - strlen(field), (int) ' ');
+	CatNcharacters (IW->iLines[(int)varN],
+			VARNAME_FIELD_LEN - strlen(field), (int) ' ');
       }
       CatNcharacters (IW->iLines[(int)varN], VARNAME_FIELD_BLANKS, (int) ' ');
       IW->iLineNs[curItem] = (int) varN;
@@ -786,7 +790,7 @@ Logical *changed;	/* Assumed to be initialized to FALSE by caller. */
      * If the variable is compressed and the values actually compressed, put
      * a "loading" message where the record/indices/value will be displayed.
      *************************************************************************/
-     if (cType != NO_COMPRESSION && cPct < 100)
+     if ((cType != NO_COMPRESSION) && (cPct < 100))
        strcpyX (vLine, "Loading values...", MAX_SCREENLINE_LEN);
      else
        MakeNUL (vLine);
@@ -813,138 +817,138 @@ Logical *changed;	/* Assumed to be initialized to FALSE by caller. */
          * Up or down arrow.
          *********************************************************************/
          case INCREMENTkey_EDIT:
-		 case DECREMENTkey_EDIT: {
-		   Logical increment = (IW.key == INCREMENTkey_EDIT);
-		   /*****************************************************************
-		   * Check for the record number.  If the record variance is FALSE,
-		   * incrementing and/or decrementing is illegal.
-		   *****************************************************************/
-		   if (IW.itemN == 0) {
-		     if (recVary) {
-		       status = CDFlib (GET_, VAR_MAXREC(Z), &maxRec,
-						        NULL_);
-		       if (!ReportStatus(status,FALSE)) {
+	 case DECREMENTkey_EDIT: {
+	   Logical increment = (IW.key == INCREMENTkey_EDIT);
+	   /*****************************************************************
+	   * Check for the record number.  If the record variance is FALSE,
+	   * incrementing and/or decrementing is illegal.
+	   *****************************************************************/
+	   if (IW.itemN == 0) {
+	     if (recVary) {
+	       status = CDFlib (GET_, VAR_MAXREC(Z), &maxRec,
+			        NULL_);
+	       if (!ReportStatus(status,FALSE)) {
 				 ItemWindow (DELETEiw, &IW);
-				 return FALSE;
-	   		   }
-		       if (maxRec == -1) maxRec = 0;
-		   	   if (increment)
+		 return FALSE;
+  	       }
+	       if (maxRec == -1) maxRec = 0;
+	       if (increment)
 #if MAXREC_LIMIT
-		         recNum = BOO(recNum < maxRec,recNum + 1,0);
+		 recNum = BOO(recNum < maxRec,recNum + 1,0);
 #else
-				 recNum++;
+	 	 recNum++;
 #endif
-		       else
+	       else
 #if MAXREC_LIMIT
-		         recNum = BOO(recNum == 0,maxRec,
-						      BOO(recNum > maxRec,maxRec,recNum - 1));
+		 recNum = BOO(recNum == 0,maxRec,
+			      BOO(recNum > maxRec,maxRec,recNum - 1));
 #else
-		         recNum = BOO(recNum == 0,maxRec,recNum - 1);
+		 recNum = BOO(recNum == 0,maxRec,recNum - 1);
 #endif
-		       if (!BuildVarValueLine(Z,recNum,recVary,numDims,indices,
+	       if (!BuildVarValueLine(Z,recNum,recVary,numDims,indices,
 				      dimVarys,dataType,numElems,binary,
 				      vString,vLine,&(IW.nItems),vvCols,vvLens,
 				      IW.nColsTotal-2,format,&overflow,nBytes)) {
-				 ItemWindow (DELETEiw, &IW);
-				 return FALSE;
-		       }
-		       ItemWindow (UPDATEiw, &IW, IW.itemN);
-		     }
-		     else
-		       ItemWindow (BEEPiw, &IW);
-		     break;
-		   }
-		   /*****************************************************************
-		   * Check for the value (which is an error - just ring bell).  The
-		   * value is always the last item.
-		   *****************************************************************/
-		   if (IW.itemN == IW.nItems - 1) {
-		     ItemWindow (BEEPiw, &IW);
-		     break;
-		   }
-		   /*****************************************************************
-		   * Must be a dimension index.  Calculate which dimension was
-		   * selected.  There must be at least one dimension for this
-		   * point to have been reached.
-		   *****************************************************************/
-		   dimN = IW.itemN - 1;
-		   if (dimVarys[dimN]) {
-		     if (increment)
-		       indices[dimN] = (indices[dimN] + 1) % dimSizes[dimN];
-		     else
-		       indices[dimN] = (indices[dimN] - 1 + dimSizes[dimN]) %
-				       dimSizes[dimN];
-		     if (!BuildVarValueLine(Z,recNum,recVary,numDims,indices,dimVarys,
-					    dataType,numElems,binary,vString,vLine,
-					    &(IW.nItems),vvCols,vvLens,
-					    IW.nColsTotal-2,format,&overflow,nBytes)) {
-		       ItemWindow (DELETEiw, &IW);
-		       return FALSE;
-		     }
-		     ItemWindow (UPDATEiw, &IW, IW.itemN);
-		   }
-		   else
-		     ItemWindow (BEEPiw, &IW);
-		   break;
-		 }
-		 /*********************************************************************
+		 ItemWindow (DELETEiw, &IW);
+		 return FALSE;
+	       }
+	       ItemWindow (UPDATEiw, &IW, IW.itemN);
+	     }
+	     else
+	       ItemWindow (BEEPiw, &IW);
+	     break;
+	   }
+	   /*****************************************************************
+	   * Check for the value (which is an error - just ring bell).  The
+	   * value is always the last item.
+	   *****************************************************************/
+	   if (IW.itemN == IW.nItems - 1) {
+	     ItemWindow (BEEPiw, &IW);
+	     break;
+	   }
+	   /*****************************************************************
+	   * Must be a dimension index.  Calculate which dimension was
+	   * selected.  There must be at least one dimension for this
+	   * point to have been reached.
+	   *****************************************************************/
+	   dimN = IW.itemN - 1;
+	   if (dimVarys[dimN]) {
+	     if (increment)
+	       indices[dimN] = (indices[dimN] + 1) % dimSizes[dimN];
+	     else
+	       indices[dimN] = (indices[dimN] - 1 + dimSizes[dimN]) %
+			       dimSizes[dimN];
+	     if (!BuildVarValueLine(Z,recNum,recVary,numDims,indices,dimVarys,
+				    dataType,numElems,binary,vString,vLine,
+				    &(IW.nItems),vvCols,vvLens,
+				    IW.nColsTotal-2,format,&overflow,nBytes)) {
+	       ItemWindow (DELETEiw, &IW);
+	       return FALSE;
+	     }
+	     ItemWindow (UPDATEiw, &IW, IW.itemN);
+	   }
+	   else
+	     ItemWindow (BEEPiw, &IW);
+	   break;
+	 }
+	 /*********************************************************************
          * Modify key.
          *********************************************************************/
          case ENTERkey_FSI:
-		   /*******************************************************************
-		   * Check for the record number.  If the record variance is FALSE, a
-		   * modification is illegal.
-		   *******************************************************************/
-		   if (IW.itemN == 0) {
-		     if (recVary) {
-		       static char *header[] = { "Enter record number..." };
-		       AOSs1 (trailer,
-			      "Enter: ________   Exit: ________  Help: ________")
+	   /*******************************************************************
+	   * Check for the record number.  If the record variance is FALSE, a
+	   * modification is illegal.
+	   *******************************************************************/
+	   if (IW.itemN == 0) {
+	     if (recVary) {
+	       static char *header[] = { "Enter record number..." };
+	       AOSs1 (trailer,
+		      "Enter: ________   Exit: ________  Help: ________")
 		       static char value[MAX_RECORD_NUMBER_LEN+1];
 		       static int exitChars[] = {
 				 ENTERkey_FSI, EXITkey_FSI, HELPkey_FSI, NUL
 		       };
-		       static struct PromptWindowStruct PW = {
-		         NULL, 9, 0, 80, 1, header, MAX_RECORD_NUMBER_LEN, value,
-		         1, trailer, exitChars, REFRESHkey_FSI, SOLkey_FSI,
-		         EOLkey_FSI, INSERTorOVERkey_FSI
-		       };
-		       EncodeKeyDefinitions (1, trailer, ENTERkey_FSI, EXITkey_FSI,
-								     HELPkey_FSI);
-		       snprintf (PW.value, (size_t) sizeof(value),
-				 "%ld", recNum + 1);
-		       PromptWindow (NEWpw, &PW, (int) strlen(PW.value), LogicalTRUE);
-		       for (;;) {
-				  if (EnterPW(&PW,RECNUMBERhelpID)) {
-				    long newRecNum;
-				    if (sscanf(PW.value,"%ld",&newRecNum) == 1) {
-				      if (newRecNum > 0) {
-				        recNum = newRecNum - 1;
-				        if (!BuildVarValueLine(Z,recNum,recVary,numDims,
-						      indices,dimVarys,dataType,
-						      numElems,binary,vString,vLine,
-						      &(IW.nItems),vvCols,vvLens,
-						      IW.nColsTotal-2,
-						      format,&overflow,nBytes)) {
-						  PromptWindow (DELETEpw, &PW);
-						  ItemWindow (DELETEiw, &IW);
-						  return FALSE;
-						}
-				        ItemWindow (UPDATEiw, &IW, IW.itemN);
-				        break;
-				      }
-				      else
-				        ProblemWindow ("Record number out of valid range.",
-								       FALSE);
-				    }
-			    else
+	       static struct PromptWindowStruct PW = {
+		      NULL, 9, 0, 80, 1, header, MAX_RECORD_NUMBER_LEN, value,
+		      1, trailer, exitChars, REFRESHkey_FSI, SOLkey_FSI,
+		      EOLkey_FSI, INSERTorOVERkey_FSI
+	       };
+	       EncodeKeyDefinitions (1, trailer, ENTERkey_FSI, EXITkey_FSI,
+				     HELPkey_FSI);
+	       snprintf (PW.value, (size_t) sizeof(value),
+			 "%ld", recNum + 1);
+	       PromptWindow (NEWpw, &PW, (int) strlen(PW.value), LogicalTRUE);
+	       for (;;) {
+		  if (EnterPW(&PW,RECNUMBERhelpID)) {
+		    long newRecNum;
+		    if (sscanf(PW.value,"%ld",&newRecNum) == 1) {
+		      if (newRecNum > 0) {
+		        recNum = newRecNum - 1;
+		        if (!BuildVarValueLine(Z,recNum,recVary,numDims,
+					       indices,dimVarys,dataType,
+					       numElems,binary,vString,vLine,
+					       &(IW.nItems),vvCols,vvLens,
+					       IW.nColsTotal-2,
+					       format,&overflow,nBytes)) {
+			  PromptWindow (DELETEpw, &PW);
+			  ItemWindow (DELETEiw, &IW);
+			  return FALSE;
+			}
+		        ItemWindow (UPDATEiw, &IW, IW.itemN);
+		        break;
+		      }
+		      else
+		        ProblemWindow ("Record number out of valid range.",
+				       FALSE);
+		    }
+		    else
 		      ProblemWindow ("Error decoding record number.", FALSE);
 		  }
 		  else
 		    break;
 	       }
 	       PromptWindow (DELETEpw, &PW);
-	       }
+	     }
 	     else
 	       ItemWindow (BEEPiw, &IW);
 	     break;
@@ -954,7 +958,7 @@ Logical *changed;	/* Assumed to be initialized to FALSE by caller. */
 	   * browsing).  The value is always the last item.
 	   *****************************************************************/
 	   if (IW.itemN == IW.nItems - 1) {
-	     if (browseOnly && !overflow) {
+	     if (browseOnly && (!overflow)) {
 	       ItemWindow (BEEPiw, &IW);
 	     }
 	     else {
@@ -1002,11 +1006,17 @@ Logical *changed;	/* Assumed to be initialized to FALSE by caller. */
 		  if (EnterPW(&PW,VARVALUEhelpID)) {
 		    long nElemsEntered; void *newBinary;
                     int style;
-                    if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
-                    else style = EPOCH0_STYLE;
+                    if (TT2000dataType(dataType)) {
+                      if (!ziso8601) style = TT2000_3_STYLE;
+                      else style = TT2000_4_STYLE;
+                    } else { 
+                      if (!ziso8601) style = DEFAULTstyleEDIT;
+                      else style = EPOCH3_STYLE;
+                    }      
 		    if (DecodeValues(PW.value,dataType,
 				     &nElemsEntered,&newBinary,style)) {
-		      if (nElemsEntered == numElems) {
+/*		      if (nElemsEntered == numElems) { */
+		      if (nElemsEntered <= numElems) {
 			status = CDFlib (PUT_, VAR_DATA(Z), newBinary,
 					 NULL_);
 			cdf_FreeMemory (newBinary, FatalError);
@@ -1086,7 +1096,7 @@ Logical *changed;	/* Assumed to be initialized to FALSE by caller. */
 		  if (EnterPW(&PW,DIMINDEXhelpID)) {
 		    long newIndex;
 		    if (sscanf(PW.value,"%ld",&newIndex) == 1) {
-		      if (newIndex > 0 && newIndex <= dimSizes[dimN]) {
+		      if ((newIndex > 0) && (newIndex <= dimSizes[dimN])) {
 		        indices[dimN] = newIndex - 1;
 		        if (!BuildVarValueLine(Z,recNum,recVary,numDims,
 					       indices,dimVarys,dataType,
@@ -1223,8 +1233,13 @@ size_t width;
    *overflow = BOO(STRINGdataType(dataType),
 		   availCols < numElems + 3,
 		   availCols < MAX_nonSTRING_VALUE_LEN);
-   if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
-   else style = EPOCH0_STYLE;
+   if (TT2000dataType(dataType)) {
+     if (!ziso8601) style = TT2000_3_STYLE;
+     else style = TT2000_4_STYLE;
+   } else { 
+     if (!ziso8601) style = DEFAULTstyleEDIT;
+     else style = EPOCH3_STYLE;
+   }      
    if ((dataType == CDF_FLOAT) || (dataType == CDF_REAL4)) {
      if (!isnan((double)*(float *)binary) && 
          !isinf((double)*(float *)binary) &&
@@ -1246,6 +1261,28 @@ size_t width;
        EncodeValuesFormat (dataType, numElems, binary, EofS(vLine), format,
                            -BOO(virtual,availCols,0), availCols, style,
                            (size_t) MAX_SCREENLINE_LEN+1-strlen(vLine));
+   } else if (dataType == CDF_EPOCH) {
+     if (!isnan(*(double *)binary) && !isinf(*(double *)binary) &&
+         *(double *)binary <= DEFAULT_EPOCH_PADVALUE) {
+       EncodeValuesFormat (dataType, numElems, binary, EofS(vLine), NULL,
+                           -BOO(virtual,availCols,0), availCols, style,
+                           (size_t) MAX_SCREENLINE_LEN+1-strlen(vLine));
+     } else
+       EncodeValuesFormat (dataType, numElems, binary, EofS(vLine), format,
+                           -BOO(virtual,availCols,0), availCols, style,
+                           (size_t) MAX_SCREENLINE_LEN+1-strlen(vLine));
+   } else if (dataType == CDF_EPOCH16) {
+     if (!isnan(*(double *)binary) && !isinf(*(double *)binary) &&
+         !isnan(*(((double *)binary)+1)) && !isinf(*(((double *)binary)+1)) &&
+         *(double *)binary <= DEFAULT_EPOCH16_PADVALUE &&
+         *(((double *)binary)+1) <= DEFAULT_EPOCH16_PADVALUE) {
+       EncodeValuesFormat (dataType, numElems, binary, EofS(vLine), NULL,
+                           -BOO(virtual,availCols,0), availCols, style,
+                           (size_t) MAX_SCREENLINE_LEN+1-strlen(vLine));
+     } else
+       EncodeValuesFormat (dataType, numElems, binary, EofS(vLine), format,
+                           -BOO(virtual,availCols,0), availCols, style,
+                           (size_t) MAX_SCREENLINE_LEN+1-strlen(vLine));
    } else
      EncodeValuesFormat (dataType, numElems, binary, EofS(vLine), format,
 		         -BOO(virtual,availCols,0), availCols, style,
@@ -1258,26 +1295,36 @@ size_t width;
          !isinf((double)*(float *)binary) &&
          *(float *)binary <= DEFAULT_FLOAT_PADVALUE) {
        EncodeValuesFormat (dataType, numElems, binary, vString, NULL, 0,
-                           BOO(STRINGdataType(dataType),
-                               (int)(numElems + 3),MAX_nonSTRING_VALUE_LEN),
-                           style, width);
+                           MAX_nonSTRING_VALUE_LEN, style, width);
      } else
        EncodeValuesFormat (dataType, numElems, binary, vString, format, 0,
-                           BOO(STRINGdataType(dataType),
-                               (int)(numElems + 3),MAX_nonSTRING_VALUE_LEN),
-                           style, width);
+                           MAX_nonSTRING_VALUE_LEN, style, width);
    } else if ((dataType == CDF_DOUBLE) || (dataType == CDF_REAL8)) {
      if (!isnan(*(double *)binary) && !isinf(*(double *)binary) &&
          *(double *)binary <= DEFAULT_DOUBLE_PADVALUE) {
        EncodeValuesFormat (dataType, numElems, binary, vString, NULL, 0,
-                           BOO(STRINGdataType(dataType),
-                               (int)(numElems + 3),MAX_nonSTRING_VALUE_LEN),
-                           style, width);
+                           MAX_nonSTRING_VALUE_LEN, style, width);
      } else
        EncodeValuesFormat (dataType, numElems, binary, vString, format, 0,
-                           BOO(STRINGdataType(dataType),
-                               (int)(numElems + 3),MAX_nonSTRING_VALUE_LEN),
-                           style, width);
+                           MAX_nonSTRING_VALUE_LEN, style, width);
+   } else if (dataType == CDF_EPOCH) {
+     if (!isnan(*(double *)binary) && !isinf(*(double *)binary) &&
+         *(double *)binary <= DEFAULT_EPOCH_PADVALUE) {
+       EncodeValuesFormat (dataType, numElems, binary, vString, NULL, 0,
+                           MAX_nonSTRING_VALUE_LEN, style, width);
+     } else
+       EncodeValuesFormat (dataType, numElems, binary, vString, format, 0,
+                           MAX_nonSTRING_VALUE_LEN, style, width);
+   } else if (dataType == CDF_EPOCH16) {
+     if (!isnan(*(double *)binary) && !isinf(*(double *)binary) &&
+         !isnan(*(((double *)binary)+1)) && !isinf(*(((double *)binary)+1)) &&
+         *(double *)binary <= DEFAULT_EPOCH16_PADVALUE &&
+         *(((double *)binary)+1) <= DEFAULT_EPOCH16_PADVALUE) {
+       EncodeValuesFormat (dataType, numElems, binary, vString, NULL, 0,
+                           MAX_nonSTRING_VALUE_LEN, style, width);
+     } else
+       EncodeValuesFormat (dataType, numElems, binary, vString, format, 0,
+                           MAX_nonSTRING_VALUE_LEN, style, width);
    } else           
      EncodeValuesFormat (dataType, numElems, binary, vString, format, 0,
 		         BOO(STRINGdataType(dataType),
@@ -1314,12 +1361,20 @@ long nVars;
      "<Allocate records> <Write initial records> <Modify compression>",
      "<Delete records>"
    };
+   static char *itemsEdit2[] = {
+     "<Modify pad value> <Modify blocking factor> <Modify sparseness>",
+     "<Allocate records> <Write initial records> <Modify compression>",
+     "<Delete records opt1> <Delete records opt2>"
+   };
    static int iLineNsBrowse[] = { 0 };
    static int iColsBrowse[] = { 0 };
    static int iLensBrowse[] = { 16 };
    static int iLineNsEdit[] = { 0,0,0,1,1,1,2 };
+   static int iLineNsEdit2[] = { 0,0,0,1,1,1,2,2 };
    static int iColsEdit[] = { 0,19,44,0,19,43,0 };
+   static int iColsEdit2[] = { 0,19,44,0,19,43,0,22 };
    static int iLensEdit[] = { 18,24,19,18,23,20,16 };
+   static int iLensEdit2[] = { 18,24,19,18,23,20,21,21 };
    static struct ItemWindowStruct IW = {
       0, 0, 80, label, 6, header, 3, NULL, 0, NULL, NULL, NULL, 3, 1, trailer,
       exitChars, REFRESHkey_FSI, FALSE, NUL, NUL
@@ -1340,11 +1395,19 @@ long nVars;
        IW.iLens = iLensBrowse;
      }
      else {
-       IW.iLines = itemsEdit;
-       IW.nItems = 7;
-       IW.iLineNs = iLineNsEdit;
-       IW.iCols = iColsEdit;
-       IW.iLens = iLensEdit;
+       if (RegularVar(Z, varN)) {
+         IW.iLines = itemsEdit;
+         IW.nItems = 7;
+         IW.iLineNs = iLineNsEdit;
+         IW.iCols = iColsEdit;
+         IW.iLens = iLensEdit;
+       } else {
+         IW.iLines = itemsEdit2;
+         IW.nItems = 8;
+         IW.iLineNs = iLineNsEdit2;
+         IW.iCols = iColsEdit2;
+         IW.iLens = iLensEdit2;
+       }
      }
    /***************************************************************************
    * Build variable `more' menu and display window.
@@ -1574,19 +1637,29 @@ long nVars;
 	   /*******************************************************************
 	   * Delete records.
 	   *******************************************************************/
-	   case DELETErecordsIN: {
-	     static char *header[] = {
-	       "Enter first/last records to delete.  Examples: 1/100, 10/10"
+	   case DELETErecords1IN:
+	   case DELETErecords2IN: {
+	     static char *header1[] = {
+	       "Enter first/last records to delete.  Example: 1/100",
+               "No record renumbering after the deletion for sparse variable."
+	     };
+	     static char *header2[] = {
+	       "Enter first/last records to delete.  Example: 1/100",
+               "Record is renumbered after the deletion for sparse variable."
 	     };
 	     AOSs1 (trailer,"Enter: ________   Exit: ________  Help: ________")
 	     static int exitChars[] = { ENTERkey_FSI, EXITkey_FSI,
 					HELPkey_FSI, NUL };
 	     static char value[MAX_nonSTRING_VALUE_LEN+1];
 	     static struct PromptWindowStruct PWt = {
-		NULL, 14, 0, 80, 1, header, MAX_nonSTRING_VALUE_LEN, value,
+		NULL, 14, 0, 80, 2, NULL, MAX_nonSTRING_VALUE_LEN, value,
 		1, trailer, exitChars, REFRESHkey_FSI, SOLkey_FSI,
 		EOLkey_FSI, INSERTorOVERkey_FSI
 	     };
+             if (IW.itemN == DELETErecords1IN)
+               PWt.hLines = header1;
+             else
+               PWt.hLines = header2;
 	     EncodeKeyDefinitions (1, trailer, ENTERkey_FSI, EXITkey_FSI,
 				   HELPkey_FSI);
 	     strcpyX (value, "", MAX_nonSTRING_VALUE_LEN);
@@ -1598,8 +1671,10 @@ long nVars;
 		    firstRecN -= 1;
 		    lastRecN -= 1;
 		    status = CDFlib (SELECT_, VAR(Z), varN,
-				     DELETE_, VAR_RECORDS(Z), firstRecN,
-							      lastRecN,
+				     DELETE_, (IW.itemN==DELETErecords1IN?
+					       VAR_RECORDS(Z):
+					       VAR_RECORDS_RENUMBER(Z)),
+					      firstRecN, lastRecN,
 				     NULL_);
 		    if (ReportStatus(status,FALSE)) {
 		      if (!BuildVarMoreMenu(Z,varN,&IW,useFormat,FALSE)) {
@@ -1759,9 +1834,9 @@ Logical center;		/* Center messages displayed? */
 	    (size_t) sizeof(BLANKs78)-strlen(IW->hLines[3]),
 	    "   Compression: %s", CompressionToken(cType,cParms));
   if (cPct == 0) cPct = 1;
-  if (cType != NO_COMPRESSION && cPct > 0) {
-    snprintf (EofS(IW->hLines[3]), (size_t) sizeof(BLANKs78), " (ratio: %ld%%)",
-              (cPct<=100?(100-cPct):0));
+  if ((cType != NO_COMPRESSION) && (cPct > 0) && (maxRec > 0)) {
+    snprintf (EofS(IW->hLines[3]), (size_t) sizeof(BLANKs78), " (saving: %d%%)",
+              (int)(cPct<=100?(100-cPct):0));
   }
   /****************************************************************************
   * Encode 5th header line.
@@ -1777,8 +1852,13 @@ Logical center;		/* Center messages displayed? */
 				  FatalError);
   status = CDFlib (GET_, VAR_PADVALUE(Z), padBinary,
 		   NULL_);
-  if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
-  else style = EPOCH0_STYLE;
+  if (TT2000dataType(dataType)) {
+    if (!ziso8601) style = TT2000_3_STYLE;
+    else style = TT2000_4_STYLE;
+  } else { 
+    if (!ziso8601) style = DEFAULTstyleEDIT;
+    else style = EPOCH3_STYLE;
+  }      
   switch (status) {
     case NO_PADVALUE_SPECIFIED: {
       static char defaultT[] = " (default)";
@@ -2051,8 +2131,13 @@ Logical *changed;	/* Assumed to be initialized to FALSE by caller. */
   }
   else
     format = NULL;
-  if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
-  else style = EPOCH0_STYLE;
+  if (TT2000dataType(dataType)) {
+    if (!ziso8601) style = TT2000_3_STYLE;
+    else style = TT2000_4_STYLE;
+  } else { 
+    if (!ziso8601) style = DEFAULTstyleEDIT;
+    else style = EPOCH3_STYLE;
+  }      
   if ((dataType == CDF_FLOAT) || (dataType == CDF_REAL4)) {
     if (!isnan((double)*(float *)binary) &&
         !isinf((double)*(float *)binary) &&
@@ -2070,6 +2155,24 @@ Logical *changed;	/* Assumed to be initialized to FALSE by caller. */
     } else
       EncodeValuesFormat (dataType, numElems, binary, PWt.value, format, 0,
                           (int) nBytes-1, style, (size_t) nBytes);
+  } else if (dataType == CDF_EPOCH) {
+    if (!isnan(*(double *)binary) && !isinf(*(double *)binary) &&
+        *(double *)binary <= DEFAULT_EPOCH_PADVALUE) {
+      EncodeValuesFormat (dataType, numElems, binary, PWt.value, NULL, 0,
+                          (int) nBytes-1, style, (size_t) nBytes);
+    } else
+      EncodeValuesFormat (dataType, numElems, binary, PWt.value, format, 0,
+                          (int) nBytes-1, style, (size_t) nBytes);
+  } else if (dataType == CDF_EPOCH16) {
+    if (!isnan(*(double *)binary) && !isinf(*(double *)binary) &&
+        !isnan(*(((double *)binary)+1)) && !isinf(*(((double *)binary)+1)) &&
+        *(double *)binary <= DEFAULT_EPOCH16_PADVALUE &&
+        *(((double *)binary)+1) <= DEFAULT_EPOCH16_PADVALUE) {
+      EncodeValuesFormat (dataType, numElems, binary, PWt.value, NULL, 0,
+                          (int) nBytes-1, style, (size_t) nBytes);
+    } else
+      EncodeValuesFormat (dataType, numElems, binary, PWt.value, format, 0,
+                          (int) nBytes-1, style, (size_t) nBytes);
   } else
     EncodeValuesFormat (dataType, numElems, binary, PWt.value, format, 0, 
 		        (int) nBytes-1, style, (size_t) nBytes);
@@ -2083,7 +2186,8 @@ Logical *changed;	/* Assumed to be initialized to FALSE by caller. */
        long nElemsEntered; void *newBinary;
        if (DecodeValues(PWt.value,dataType,
 			&nElemsEntered,&newBinary,style)) {
-	 if (nElemsEntered == numElems) {
+/*	 if (nElemsEntered == numElems) { */
+	 if (nElemsEntered <= numElems) {
 	   status = CDFlib (PUT_, VAR_PADVALUE(zVar), newBinary,
 			    NULL_);
 	   if (ReportStatus(status,FALSE)) {
@@ -2116,3 +2220,21 @@ Logical *changed;	/* Assumed to be initialized to FALSE by caller. */
      }
   }
 }
+
+/******************************************************************************
+* ModPadValue.
+******************************************************************************/
+
+static Logical RegularVar (zVar, varN)
+Logical zVar;
+long varN;
+{
+  long sr;
+  CDFstatus status;
+  status = CDFlib (SELECT_, VAR(zVar), varN,
+		   GET_, VAR_SPARSERECORDS(zVar), &sr,
+		   NULL_);
+  if (!ReportStatus(status,FALSE)) return FALSE;
+  if (sr == NO_SPARSERECORDS) return TRUE;
+  else return FALSE;
+}
