@@ -51,7 +51,8 @@ extern unsigned _stklen = BORLANDC_STACK_SIZE;
 * Local function prototypes.
 ******************************************************************************/
 
-static char *epochToken PROTOARGs((int zMode));
+static char *epochToken PROTOARGs((int epoch));
+static char *tt2000Token PROTOARGs((int tt2000));
 static char *majorityToken PROTOARGs((int zMode));
 static char *zModeToken PROTOARGs((int zMode));
 static char *No PROTOARGs((Logical state));
@@ -96,11 +97,12 @@ int argC;
 char *argV[];
 {
   QOP *qop;
-  static char *validQuals[] = { "id", "page", "nopage", "about", NULL };
-  static int optRequired[] = { FALSE, FALSE, FALSE, FALSE, 0 };
+  static char *validQuals[] = { "id", "page", "nopage", "about", "ziso8601", 
+                                NULL };
+  static int optRequired[] = { FALSE, FALSE, FALSE, FALSE, FALSE, 0 };
   Logical qopError = FALSE;
 
-  CDFsetValidate (VALIDATEFILEon);
+/*  CDFsetValidate (VALIDATEFILEon); */
   /****************************************************************************
   * Based on the number of command line arguments entered...
   ****************************************************************************/
@@ -137,6 +139,14 @@ char *argV[];
       ***********************************************************************/
       qopError = qopError | !TFqualifier(qop,&pagingOn,PAGEqual,NOPAGEqual,
 					 DEFAULTpageINQ,"page");
+      /*************************************************************************
+      * Check for /ZISO8601,-ziso8601 qualifier.
+      *************************************************************************/
+      if (qop->qualEntered[ZISO8601qual]) {
+        ziso8601 = TRUE;
+      }
+      else
+        ziso8601 = FALSE;
       /************************************************************************
       * Free QOP memory and check for an error.
       ************************************************************************/
@@ -227,10 +237,11 @@ void DisplayBlockingDefaults () {
 
 void DisplayPadValueDefaults () {
   char text[MAX_SCREENLINE_LEN+1];
-  char epString[EPOCH_STRING_LEN+1];
-  char epString16[EPOCH16_STRING_LEN+1];
   char tt2000[TT2000_3_STRING_LEN+1];
-  double epoch16[2];
+  double dummy, epoch16[2];
+  char *epString;
+  int style;
+
   WriteOutSO ("\nDefault pad values:\n\n");
   snprintf (text, (size_t) sizeof(text),
 	    "  CDF_BYTE: %d\n", (int) DEFAULT_BYTE_PADVALUE);
@@ -280,17 +291,24 @@ void DisplayPadValueDefaults () {
   snprintf (text, (size_t) sizeof(text),
 	    "  CDF_DOUBLE: %g\n", (double) DEFAULT_DOUBLE_PADVALUE);
   WriteOutSO (text);
-  encodeEPOCH((double) DEFAULT_EPOCH_PADVALUE, epString);
+  dummy = (double) DEFAULT_EPOCH_PADVALUE;
+  if (!ziso8601) style = DEFAULTstyleINQ;
+  else style = EPOCH3_STYLE;
+  epString = EpochDateTimeString (CDF_EPOCH, style, &dummy);
   snprintf (text, (size_t) sizeof(text),
-	    "  CDF_EPOCH: %s\n", epString);
+            "  CDF_EPOCH: %s\n", epString);
+  if (epString != NULL) free (epString);
   WriteOutSO (text);
   epoch16[0] = epoch16[1] = DEFAULT_EPOCH16_PADVALUE;
-  encodeEPOCH16(epoch16, epString16);
+  epString = EpochDateTimeString (CDF_EPOCH16, style, epoch16);
   snprintf (text, (size_t) sizeof(text),
-	    "  CDF_EPOCH16: %s\n", epString16);
+            "  CDF_EPOCH16: %s\n", epString);
+  if (epString != NULL) free (epString);
   WriteOutSO (text);
+  if (!ziso8601) style = TT2000_3_STYLE;
+  else style = TT2000_4_STYLE;
   CDF_TT2000_to_UTC_string((Int64) DEFAULT_TT2000_PADVALUE, tt2000,
-                           TT2000_3_STYLE);
+                           style);
   snprintf (text, (size_t) sizeof(text),
             "  CDF_TIME_TT2000: %s\n", tt2000);
   WriteOutSO (text);
@@ -362,15 +380,15 @@ void DisplayMiscDefaults () {
 	    "Default format: %s\n",
 	   BOO((Logical)DEFAULT_TO_SINGLE_FILE,"SINGLE_FILE","MULTI_FILE"));
   WriteOutSO (text);
-#if defined(alphavmsD)
+#if defined(alphavmsD) || defined(ia64vmsD)
   WriteOutSO ("Default single-precision floating-point: F_FLOAT\n");
   WriteOutSO ("Default double-precision floating-point: D_FLOAT\n");
 #endif
-#if defined(alphavmsG)
+#if defined(alphavmsG) || defined(ia64vmsG)
   WriteOutSO ("Default single-precision floating-point: F_FLOAT\n");
   WriteOutSO ("Default double-precision floating-point: G_FLOAT\n");
 #endif
-#if defined(alphavmsI)
+#if defined(alphavmsI) || defined(ia64vmsI)
   WriteOutSO ("Default single-precision floating-point: IEEE_FLOAT\n");
   WriteOutSO ("Default double-precision floating-point: IEEE_FLOAT\n");
 #endif
@@ -935,8 +953,9 @@ static void CDFexportInitialVMS () {
 	    No(DEFAULToutputEXPORT), No(DEFAULTdeleteEXPORT));
   WriteOut (stdout, text);
   snprintf (text, (size_t) sizeof(text),
-	    "              %sEXCLUSIVE,%s,%s,%s,%s,%s%sHEADING)\n",
-	    No(DEFAULTexclusiveEXPORT), epochToken(DEFAULTepochEXPORT),
+	    "              %sEXCLUSIVE,%s,%s,%s,%s,%s,%s%sHEADING)\n",
+	    No(DEFAULTexclusiveEXPORT), epochToken(DEFAULTstyleEXPORT),
+            epochToken(DEFAULTtt2000EXPORT),
 	    BOO(DEFAULTsingleEXPORT,"SINGLE","MULTI"),
 	    BOO(DEFAULTnetworkEXPORT,"NETWORK","HOST"),
 	    BOO(DEFAULThorizontalEXPORT,"HORIZONTAL","VERTICAL"),
@@ -959,8 +978,9 @@ static void CDFsimpleInitialVMS () {
 	    No(DEFAULTindicesSIMPLE), No(DEFAULToutputSIMPLE));
   WriteOut (stdout, text);
   snprintf (text, (size_t) sizeof(text),
-	    "              %s,%s,%s%sHEADING)\n",
+	    "              %s,%s,%s,%s%sHEADING)\n",
 	    epochToken(DEFAULTepochSIMPLE),
+            epochToken(DEFAULTtt2000SIMPLE),
 	    BOO(DEFAULThorizontalSIMPLE,"HORIZONTAL","VERTICAL"),
 	    majorityToken(DEFAULTmajoritySIMPLE), No(DEFAULTheadingSIMPLE));
   WriteOut (stdout, text);
@@ -1033,8 +1053,9 @@ static void CDFexportInitialNotVMS () {
 	    No(DEFAULToutputEXPORT), No(DEFAULTdeleteEXPORT));
   WriteOut (stdout, text);
   snprintf (text, (size_t) sizeof(text),
-	    "              %sexclusive,%s,%s,%s,%s,%s%sheading\"\n",
-	    No(DEFAULTexclusiveEXPORT), epochToken(DEFAULTepochEXPORT),
+	    "              %sexclusive,%s,%s,%s,%s,%s,%s%sheading\"\n",
+	    No(DEFAULTexclusiveEXPORT), epochToken(DEFAULTstyleEXPORT),
+            epochToken(DEFAULTtt2000EXPORT),
 	    BOO(DEFAULTsingleEXPORT,"single","multi"),
 	    BOO(DEFAULTnetworkEXPORT,"network","host"),
 	    BOO(DEFAULThorizontalEXPORT,"horizontal","vertical"),
@@ -1057,8 +1078,9 @@ static void CDFsimpleInitialNotVMS () {
 	    No(DEFAULTindicesSIMPLE), No(DEFAULToutputSIMPLE));
   WriteOut (stdout, text);
   snprintf (text, (size_t) sizeof(text),
-	    "              %s,%s,%s%sheading\"\n",
+	    "              %s,%s,%s,%s%sheading\"\n",
 	    epochToken(DEFAULTepochSIMPLE),
+            epochToken(DEFAULTtt2000SIMPLE),
 	    BOO(DEFAULThorizontalSIMPLE,"horizontal","vertical"),
 	    majorityToken(DEFAULTmajoritySIMPLE), No(DEFAULTheadingSIMPLE));
   WriteOut (stdout, text);
@@ -1137,6 +1159,12 @@ int epochStyle;
       return "EPOCH3";
 #else
       return "epoch3";
+#endif
+    case EPOCH4_STYLE:
+#if defined(vms)
+      return "EPOCH4";
+#else
+      return "epoch4";
 #endif
     case EPOCHf_STYLE:
 #if defined(vms)
@@ -1154,6 +1182,48 @@ int epochStyle;
   return "?";
 }
 
+/******************************************************************************
+* tt2000Token.
+******************************************************************************/
+
+static char *tt2000Token (tt2000Style)
+int tt2000Style;
+{
+  switch (tt2000Style) {
+    case TT2000_0_STYLE:
+#if defined(vms)
+      return "TT2000_0";
+#else
+      return "tt2000_0";
+#endif
+    case TT2000_1_STYLE:
+#if defined(vms)
+      return "TT2000_1";
+#else
+      return "tt2000_1";
+#endif
+    case TT2000_2_STYLE:
+#if defined(vms)
+      return "TT2000_2";
+#else
+      return "tt2000_2";
+#endif
+    case TT2000_3_STYLE:
+#if defined(vms)
+      return "TT2000_3";
+#else
+      return "tt2000_3";
+#endif
+    case TT2000_4_STYLE:
+#if defined(vms)
+      return "TT2000_4";
+#else
+      return "tt2000_4";
+#endif
+  }
+  return "?";
+}
+
 /******************************************************************************
 * No.
 ******************************************************************************/
