@@ -31,16 +31,19 @@
 * DISPstat.
 ******************************************************************************/
 
-void DISPstat (Var)
+void DISPstat (Var, virtual)
 struct VarStruct *Var;
+int virtual;
 {
   char line[MAX_SCREENLINE_LEN+1];
 
-  DisplayMin (Var);
-  DisplayMax (Var);
-  DisplayFill (Var);
-
+  if (!epochMonotonic) {
+    DisplayMin (Var);
+    DisplayMax (Var);
+    DisplayFill (Var);
+  }
   strcpyX (line, "       monotonic: ", MAX_SCREENLINE_LEN);
+
   if (Var->checkMonotonicVar)
     if (Var->monoInited)
       switch (Var->monoState) {
@@ -69,7 +72,8 @@ struct VarStruct *Var;
           break;
       }
     else
-      strcatX (line, "n/a (all fill values)", MAX_SCREENLINE_LEN);
+      strcatX (line, (virtual==1?"n/a (0 or 1 record)":"n/a (all fill values)"),
+	       MAX_SCREENLINE_LEN);
   else
     strcatX (line, "n/a", MAX_SCREENLINE_LEN);
   strcatX (line, "\n\n", MAX_SCREENLINE_LEN);
@@ -89,8 +93,13 @@ struct VarStruct *Var;
   int style;
 
   strcpyX (line, "         minimum: ", MAX_SCREENLINE_LEN);
-  if (TT2000dataType(Var->dataTypeV)) style = TT2000_3_STYLE;
-  else style = EPOCH0_STYLE;
+  if (TT2000dataType(Var->dataTypeV)) {
+    if (!ziso8601) style = TT2000_3_STYLE;
+    else style = TT2000_4_STYLE;
+  } else {
+    if (!ziso8601) style = DEFAULTstyleSTATS;
+    else style = EPOCH3_STYLE;
+  }
   if (Var->minmaxInited) {
     if ((Var->dataTypeV == CDF_FLOAT) || (Var->dataTypeV == CDF_REAL4)) {
       if (!isnan((double)*(float *)Var->min) &&
@@ -116,6 +125,32 @@ struct VarStruct *Var;
                             &line[strlen(line)], Var->format, 0,
                             MAX_SCREENLINE_LEN - strlen(line), style,
                             (size_t) sizeof(line)-strlen(line));
+    } else if (Var->dataTypeV == CDF_EPOCH) {
+      if (!isnan(*(double *)Var->min) &&
+          *(double *)Var->min <= DEFAULT_EPOCH_PADVALUE) {
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->min,
+		            &line[strlen(line)], Var->format, 0, /* or NULL, 0 */
+		            MAX_SCREENLINE_LEN - strlen(line), style,
+		    	    (size_t) sizeof(line)-strlen(line));
+      } else
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->min,
+                            &line[strlen(line)], Var->format, 0,
+                            MAX_SCREENLINE_LEN - strlen(line), style,
+                            (size_t) sizeof(line)-strlen(line));
+    } else if (Var->dataTypeV == CDF_EPOCH16) {
+      if (!isnan(*(double *)Var->min) &&
+          !isnan(*(((double *)Var->min)+1)) &&
+          *(double *)Var->min <= DEFAULT_EPOCH16_PADVALUE &&
+          *(((double *)Var->min)+1) <= DEFAULT_EPOCH16_PADVALUE) {
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->min,
+		            &line[strlen(line)], Var->format, 0, /* or NULL, 0 */
+		            MAX_SCREENLINE_LEN - strlen(line), style,
+		    	    (size_t) sizeof(line)-strlen(line));
+      } else
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->min,
+                            &line[strlen(line)], Var->format, 0,
+                            MAX_SCREENLINE_LEN - strlen(line), style,
+                            (size_t) sizeof(line)-strlen(line));
     } else
       EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->min,
 		          &line[strlen(line)], Var->format, 0,
@@ -154,6 +189,32 @@ struct VarStruct *Var;
                                 &line[strlen(line)], Var->format, 0,
                                 MAX_SCREENLINE_LEN - strlen(line), style,
                                 (size_t) sizeof(line)-strlen(line));
+        } else if (Var->dataTypeV == CDF_EPOCH) {
+          if (!isnan(*(double *)Var->minINrange) &&
+              *(double *)Var->minINrange <= DEFAULT_EPOCH_PADVALUE) {
+            EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->minINrange,
+			        &line[strlen(line)], NULL, 0,
+			        MAX_SCREENLINE_LEN - strlen(line), style,
+			        (size_t) sizeof(line)-strlen(line));
+          } else
+            EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->minINrange,
+                                &line[strlen(line)], Var->format, 0,
+                                MAX_SCREENLINE_LEN - strlen(line), style,
+                                (size_t) sizeof(line)-strlen(line));
+        } else if (Var->dataTypeV == CDF_EPOCH16) {
+          if (!isnan(*(double *)Var->minINrange) &&
+              !isnan(*(((double *)Var->minINrange)+1)) &&
+              *(double *)Var->minINrange <= DEFAULT_EPOCH16_PADVALUE &&
+              *(((double *)Var->minINrange)+1) <= DEFAULT_EPOCH16_PADVALUE) {
+            EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->minINrange,
+			        &line[strlen(line)], NULL, 0,
+			        MAX_SCREENLINE_LEN - strlen(line), style,
+			        (size_t) sizeof(line)-strlen(line));
+          } else
+            EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->minINrange,
+                                &line[strlen(line)], Var->format, 0,
+                                MAX_SCREENLINE_LEN - strlen(line), style,
+                                (size_t) sizeof(line)-strlen(line));
         } else
           EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->minINrange,
 			      &line[strlen(line)], Var->format, 0,
@@ -191,7 +252,32 @@ struct VarStruct *Var;
                             &line[strlen(line)], Var->format, 0,
                             MAX_SCREENLINE_LEN - strlen(line), style,
                             (size_t) sizeof(line)-strlen(line));
-
+    } else if (Var->dataTypeV == CDF_EPOCH) {
+      if (!isnan(*(double *)Var->validmin) &&
+          *(double *)Var->validmin <= DEFAULT_EPOCH_PADVALUE) {
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->validmin,
+		            &line[strlen(line)], NULL, 0,
+		            MAX_SCREENLINE_LEN - strlen(line), style,
+		  	    (size_t) sizeof(line)-strlen(line));
+      } else
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->validmin,
+                            &line[strlen(line)], Var->format, 0,
+                            MAX_SCREENLINE_LEN - strlen(line), style,
+                            (size_t) sizeof(line)-strlen(line));
+    } else if (Var->dataTypeV == CDF_EPOCH16) {
+      if (!isnan(*(double *)Var->validmin) &&
+          !isnan(*(((double *)Var->validmin)+1)) &&
+          *(double *)Var->validmin <= DEFAULT_EPOCH16_PADVALUE &&
+          *(((double *)Var->validmin)+1) <= DEFAULT_EPOCH16_PADVALUE) {
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->validmin,
+		            &line[strlen(line)], NULL, 0,
+		            MAX_SCREENLINE_LEN - strlen(line), style,
+		  	    (size_t) sizeof(line)-strlen(line));
+      } else
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->validmin,
+                            &line[strlen(line)], Var->format, 0,
+                            MAX_SCREENLINE_LEN - strlen(line), style,
+                            (size_t) sizeof(line)-strlen(line));
     } else
       EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->validmin,
 		          &line[strlen(line)], Var->format, 0,
@@ -217,8 +303,13 @@ struct VarStruct *Var;
   int style;
 
   strcpyX (line, "         maximum: ", MAX_SCREENLINE_LEN);
-  if (TT2000dataType(Var->dataTypeV)) style = TT2000_3_STYLE;
-  else style = EPOCH0_STYLE;
+  if (TT2000dataType(Var->dataTypeV)) {
+    if (!ziso8601) style = TT2000_3_STYLE;
+    else style = TT2000_4_STYLE;
+  } else {
+    if (!ziso8601) style = DEFAULTstyleSTATS;
+    else style = EPOCH3_STYLE;
+  }
   if (Var->minmaxInited) {
     if ((Var->dataTypeV == CDF_FLOAT) || (Var->dataTypeV == CDF_REAL4)) {
      if (!isnan((double)*(float *)Var->max) &&
@@ -244,6 +335,32 @@ struct VarStruct *Var;
                             &line[strlen(line)], Var->format, 0,
                             MAX_SCREENLINE_LEN - strlen(line), style,
                             (size_t) sizeof(line)-strlen(line));
+    } else if (Var->dataTypeV == CDF_EPOCH) {
+      if (!isnan(*(double *)Var->max) &&
+          *(double *)Var->max <= DEFAULT_EPOCH_PADVALUE) {
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->max,
+		            &line[strlen(line)], NULL, 0,
+		            MAX_SCREENLINE_LEN - strlen(line), style,
+			    (size_t) sizeof(line)-strlen(line));
+      } else
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->max,
+                            &line[strlen(line)], Var->format, 0,
+                            MAX_SCREENLINE_LEN - strlen(line), style,
+                            (size_t) sizeof(line)-strlen(line));
+    } else if (Var->dataTypeV == CDF_EPOCH16) {
+      if (!isnan(*(double *)Var->max) &&
+          !isnan(*(((double *)Var->max)+1)) &&
+          *(double *)Var->max <= DEFAULT_EPOCH16_PADVALUE &&
+          *(((double *)Var->max)+1) <= DEFAULT_EPOCH16_PADVALUE) {
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->max,
+		            &line[strlen(line)], NULL, 0,
+		            MAX_SCREENLINE_LEN - strlen(line), style,
+			    (size_t) sizeof(line)-strlen(line));
+      } else
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->max,
+                            &line[strlen(line)], Var->format, 0,
+                            MAX_SCREENLINE_LEN - strlen(line), style,
+                            (size_t) sizeof(line)-strlen(line));
     } else
       EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->max,
 		          &line[strlen(line)], Var->format, 0,
@@ -282,6 +399,32 @@ struct VarStruct *Var;
                               &line[strlen(line)], Var->format, 0,
                               MAX_SCREENLINE_LEN - strlen(line), style,
                               (size_t) sizeof(line)-strlen(line));
+        } else if (Var->dataTypeV == CDF_EPOCH) {
+          if (!isnan(*(double *)Var->maxINrange) &&
+              *(double *)Var->maxINrange <= DEFAULT_EPOCH_PADVALUE) {
+            EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->maxINrange,
+			        &line[strlen(line)], NULL, 0,
+			        MAX_SCREENLINE_LEN - strlen(line), style,
+			        (size_t) sizeof(line)-strlen(line));
+          } else
+            EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->maxINrange,
+                              &line[strlen(line)], Var->format, 0,
+                              MAX_SCREENLINE_LEN - strlen(line), style,
+                              (size_t) sizeof(line)-strlen(line));
+        } else if (Var->dataTypeV == CDF_EPOCH16) {
+          if (!isnan(*(double *)Var->maxINrange) &&
+              !isnan(*(((double *)Var->maxINrange)+1)) &&
+              *(double *)Var->maxINrange <= DEFAULT_EPOCH16_PADVALUE &&
+              *(((double *)Var->maxINrange)+1) <= DEFAULT_EPOCH16_PADVALUE) {
+            EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->maxINrange,
+			        &line[strlen(line)], NULL, 0,
+			        MAX_SCREENLINE_LEN - strlen(line), style,
+			        (size_t) sizeof(line)-strlen(line));
+          } else
+            EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->maxINrange,
+                              &line[strlen(line)], Var->format, 0,
+                              MAX_SCREENLINE_LEN - strlen(line), style,
+                              (size_t) sizeof(line)-strlen(line));
 	} else
           EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->maxINrange,
 			      &line[strlen(line)], Var->format, 0,
@@ -307,7 +450,6 @@ struct VarStruct *Var;
                             &line[strlen(line)], Var->format, 0,
                             MAX_SCREENLINE_LEN - strlen(line), style,
                             (size_t) sizeof(line)-strlen(line));
-
     } else if ((Var->dataTypeV == CDF_DOUBLE) || (Var->dataTypeV == CDF_REAL8)) {
       if (!isnan(*(double *)Var->validmax) &&
           *(double *)Var->validmax <= DEFAULT_DOUBLE_PADVALUE) {
@@ -320,7 +462,32 @@ struct VarStruct *Var;
                             &line[strlen(line)], Var->format, 0,
                             MAX_SCREENLINE_LEN - strlen(line), style,
                             (size_t) sizeof(line)-strlen(line));
-
+    } else if (Var->dataTypeV == CDF_EPOCH) {
+      if (!isnan(*(double *)Var->validmax) &&
+          *(double *)Var->validmax <= DEFAULT_EPOCH_PADVALUE) {
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->validmax,
+		            &line[strlen(line)], NULL, 0,
+		            MAX_SCREENLINE_LEN - strlen(line), style,
+		  	    (size_t) sizeof(line)-strlen(line));
+      } else
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->validmax,
+                            &line[strlen(line)], Var->format, 0,
+                            MAX_SCREENLINE_LEN - strlen(line), style,
+                            (size_t) sizeof(line)-strlen(line));
+    } else if (Var->dataTypeV == CDF_EPOCH16) {
+      if (!isnan(*(double *)Var->validmax) &&
+          !isnan(*(((double *)Var->validmax)+1)) &&
+          *(double *)Var->validmax <= DEFAULT_EPOCH16_PADVALUE &&
+          *(((double *)Var->validmax)+1) <= DEFAULT_EPOCH16_PADVALUE) {
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->validmax,
+		            &line[strlen(line)], NULL, 0,
+		            MAX_SCREENLINE_LEN - strlen(line), style,
+		  	    (size_t) sizeof(line)-strlen(line));
+      } else
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->validmax,
+                            &line[strlen(line)], Var->format, 0,
+                            MAX_SCREENLINE_LEN - strlen(line), style,
+                            (size_t) sizeof(line)-strlen(line));
     } else
       EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->validmax,
 		          &line[strlen(line)], Var->format, 0,
@@ -347,8 +514,13 @@ struct VarStruct *Var;
 
   if (Var->ignoreFillsVar) {
     strcpyX (line, "      fill value: ", MAX_SCREENLINE_LEN);
-    if (TT2000dataType(Var->dataTypeV)) style = TT2000_3_STYLE;
-    else style = EPOCH0_STYLE;
+    if (TT2000dataType(Var->dataTypeV)) {
+      if (!ziso8601) style = TT2000_3_STYLE;
+      else style = TT2000_4_STYLE;
+    } else {
+      if (!ziso8601) style = DEFAULTstyleSTATS;
+      else style = EPOCH3_STYLE;
+    }
     if ((Var->dataTypeV == CDF_FLOAT) || (Var->dataTypeV == CDF_REAL4)) {
       if (!isnan((double)*(float *)Var->fillval) &&
           *(float *)Var->fillval <= DEFAULT_FLOAT_PADVALUE) {
@@ -373,6 +545,32 @@ struct VarStruct *Var;
                             &line[strlen(line)], Var->format, 0,
                             MAX_SCREENLINE_LEN - strlen(line), style,
                             (size_t) sizeof(line)-strlen(line));
+    } else if (Var->dataTypeV == CDF_EPOCH) {
+      if (!isnan(*(double *)Var->fillval) &&
+          *(double *)Var->fillval <= DEFAULT_EPOCH_PADVALUE) {
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->fillval,
+		            &line[strlen(line)], NULL, 0,
+		            MAX_SCREENLINE_LEN - strlen(line), style,
+		    	    (size_t) sizeof(line)-strlen(line));
+      } else
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->fillval,
+                            &line[strlen(line)], Var->format, 0,
+                            MAX_SCREENLINE_LEN - strlen(line), style,
+                            (size_t) sizeof(line)-strlen(line));
+    } else if (Var->dataTypeV == CDF_EPOCH16) {
+      if (!isnan(*(double *)Var->fillval) &&
+          !isnan(*(((double *)Var->fillval)+1)) &&
+          *(double *)Var->fillval <= DEFAULT_EPOCH16_PADVALUE && 
+          *(((double *)Var->fillval)+1) <= DEFAULT_EPOCH16_PADVALUE) {
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->fillval,
+		            &line[strlen(line)], NULL, 0,
+		            MAX_SCREENLINE_LEN - strlen(line), style,
+		    	    (size_t) sizeof(line)-strlen(line));
+      } else
+        EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->fillval,
+                            &line[strlen(line)], Var->format, 0,
+                            MAX_SCREENLINE_LEN - strlen(line), style,
+                            (size_t) sizeof(line)-strlen(line));
     } else    
       EncodeValuesFormat (Var->dataTypeV, Var->numElemsV, Var->fillval,
 		          &line[strlen(line)], Var->format, 0,
