@@ -88,7 +88,7 @@ char *argV[];
   char oSpec[DU_MAX_PATH_LEN+1], oDir[DU_MAX_DIR_LEN+1];
   char oName[DU_MAX_NAME_LEN+1], CDFpath[DU_MAX_PATH_LEN+1];
   char CDFdir[DU_MAX_DIR_LEN+1], CDFname[DU_MAX_NAME_LEN+1];
-  long numVars, numZvars, varN, zMode;
+  long numVars, numZvars, varN, zMode = DEFAULTzModeSTATS;
   long workingCache, stageCache, compressCache;
   Logical qopError = FALSE;
   struct VarStruct Var;
@@ -99,12 +99,12 @@ char *argV[];
     "page", "nopage", "update_valids", "noupdate_valids", "update_scales",
     "noupdate_scales", "zmode", "neg2posfp0", "noneg2posfp0",
     "update_monotonic", "noupdate_monotonic", "report", "cache", "about",
-    "statistics", "nostatistics", "epoch_monotonic", NULL
+    "statistics", "nostatistics", "epoch_monotonic", "ziso8601", NULL
   };
   static int optRequired[] = {
     FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE,
     FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE,
-    FALSE, FALSE, FALSE, FALSE, 0
+    FALSE, FALSE, FALSE, FALSE, FALSE, 0
   };
   static char *reportTokens[] = { "errors", "warnings", "informationals" };
 
@@ -243,6 +243,14 @@ char *argV[];
         epochMonotonic = TRUE;
       } else
         epochMonotonic = FALSE;
+      /*************************************************************************
+      * Check for /ZISO8601,-zisO8601 qualifier.
+      *************************************************************************/
+      if (qop->qualEntered[ZISO8601qual]) {
+        ziso8601 = TRUE;
+      }
+      else
+        ziso8601 = FALSE;
       /************************************************************************
       * Free QOP memory and check for an error.
       ************************************************************************/
