@@ -71,6 +71,7 @@ char *stream, *token, *string, *eValue, *location;
 size_t streamZ, tokenZ, stringZ, eValueZ, locationZ;
 char epoch[EPOCH_STRING_LEN+1];
 char epoch2[EPOCH16_STRING_LEN+1];
+char tt2000[TT2000_4_STRING_LEN+1];
 char charStr[MAX_SKTLINE_LEN+1+1];
 char charDelimiter;
 int  delimiterNum, numDelimiters;
@@ -90,6 +91,7 @@ Logical report[3];
 Logical displayStats;
 long zMode;
 long workingCache, stageCache, compressCache;
+int lineLen, lineIdx, lineIds;
 static char creCDF[] = "creating the CDF";
 static char creG[] = "creating a gAttribute";
 static char creV[] = "creating a vAttribute";
@@ -399,8 +401,10 @@ char *argV[];
   ****************************************************************************/
   NextSearchItem (TOKEN, HEADERmark);
   while (fgets(line,MAX_SKTLINE_LEN,SKTfp) != NULL) {
+    lineLen = (int) strlen(line) - 1;
     lineN++;
     for (i = 0, comment = FALSE; line[i] != NUL && !comment; i++) {
+       lineIdx = i;
        switch (line[i]) {
 	 /*********************************************************************
 	 * Newline - change to a blank.
@@ -433,6 +437,8 @@ char *argV[];
 		NextSearchItem (TOKEN, CDFCOMPRESSIONmark);
              } else if (strstrIgCase(line, "cdf_checksum") != NULL) {
 		NextSearchItem (TOKEN, CDFCHECKSUMmark);
+             } else if (strstrIgCase(line, "cdf_leapsecondlastupdated") != NULL) {
+		NextSearchItem (TOKEN, CDFLEAPSECONDLASTUPDATEDmark);
              } else if (strstrIgCase(line, "var_compression") != NULL) {
 		NextSearchItem (TOKEN, VARCOMPRESSIONmark);
              } else if (strstrIgCase(line, "var_sparserecords") != NULL) {
@@ -654,13 +660,18 @@ int chr;
 	if (!Spacing(chr)) {
 	  catchrX (epoch, chr, EPOCH_STRING_LEN);
 	  epochCount = 1;
+	  lineIds = lineIdx;
 	}
       }
       else {
 	catchrX (epoch, chr, EPOCH_STRING_LEN);
 	epochCount++;
-	if (epochCount == EPOCH_STRING_LEN)
-	  if (!ItemStream(epoch)) return FALSE;
+        if (epochCount == 3 && StrStrIgCaseX(epoch, "nan")) {
+          if (!ItemStream(epoch)) return FALSE;
+        } else {
+	  if ((lineIds+epochCount) == lineLen)
+	    if (!ItemStream(epoch)) return FALSE;
+        }
       }
       break;
     /**************************************************************************
@@ -671,13 +682,36 @@ int chr;
         if (!Spacing(chr)) {
           catchrX (epoch2, chr, EPOCH16_STRING_LEN);
           epochCount = 1;
+	  lineIds = lineIdx;
         }
       }
       else {
         catchrX (epoch2, chr, EPOCH16_STRING_LEN);
         epochCount++;
-        if (epochCount == EPOCH16_STRING_LEN)
+        if (epochCount == 3 && StrStrIgCaseX(epoch2, "nan")) {
           if (!ItemStream(epoch2)) return FALSE;
+        } else {
+	  if ((lineIds+epochCount) == lineLen)
+            if (!ItemStream(epoch2)) return FALSE;
+	}
+      }
+      break;
+    /**************************************************************************
+    *
+    **************************************************************************/
+    case VARTT2000:
+      if (epochCount == 0) {
+        if (!Spacing(chr)) {
+          catchrX (tt2000, chr, TT2000_4_STRING_LEN);
+          epochCount = 1;
+	  lineIds = lineIdx;
+        }
+      }
+      else {
+        catchrX (tt2000, chr, TT2000_4_STRING_LEN);
+        epochCount++;
+	if ((lineIds+epochCount) == lineLen)
+          if (!ItemStream(tt2000)) return FALSE;
       }
       break;
     /**************************************************************************
@@ -727,7 +761,7 @@ int chr;
 Logical ItemStream (item)
 char *item;
 {
-  CDFstatus status; Logical zVar;
+  CDFstatus status; Logical zVar = TRUE;
   static long format, majority, encoding, zNumDims, rNumDims, entryNum,
 	      vDataType, eDataType, vNumElems, recVary, dimVarys[CDF_MAX_DIMS],
 	      zDimSizes[CDF_MAX_DIMS], rDimSizes[CDF_MAX_DIMS];
@@ -736,8 +770,8 @@ char *item;
   static long VARYdimVarys[CDF_MAX_DIMS] = {
     VARY, VARY, VARY, VARY, VARY, VARY, VARY, VARY, VARY, VARY
   };
-  long numDims, dimSizes[CDF_MAX_DIMS], varNum;
-  long *dimVarysP;	/* Points to the `dimVarys' array to be used. */
+  long numDims = 0, varNum, dimSizes[CDF_MAX_DIMS];
+  long *dimVarysP = NULL;  /* Points to the `dimVarys' array to be used. */
   int dN, dNt;		/* Dimension number. */
   void *padValue;
   size_t nBytes;
@@ -1164,6 +1198,46 @@ char *item;
     /**************************************************************************
     *
     **************************************************************************/
+    case CDFLEAPSECONDLASTUPDATEDmark:
+      if (!strcmp(item,"CDF_LEAPSECONDLASTUPDATED:"))
+        NextSearchItem (TOKEN, CDFLEAPSECONDLASTUPDATEDfield);
+      else
+        if (!strcmp(item,"CDF_LEAPSECONDLASTUPDATED"))
+          NextSearchItem (TOKEN, CDFLEAPSECONDLASTUPDATEDCOLONmark);
+        else {
+          ItemError2 ("CDF_LEAPSECONDLASTUPDATED:", "CDF_LEAPSECONDLASTUPDATED", item);
+          return FALSE;
+        }
+      break;
+    /**************************************************************************
+    *
+    **************************************************************************/
+    case CDFLEAPSECONDLASTUPDATEDCOLONmark:
+      if (!strcmp(item,":"))
+	NextSearchItem (TOKEN, CDFLEAPSECONDLASTUPDATEDfield);
+      else {
+	ItemError1 (":", item);
+	return FALSE;
+      }
+      break;
+    /**************************************************************************
+    *
+    **************************************************************************/
+    case CDFLEAPSECONDLASTUPDATEDfield: {
+      long lastUpdated;
+      if (sscanf(item,"%ld",&lastUpdated) != 1) {
+	ItemError1 ("<leapsecondlastupdated>", item);
+	return FALSE;
+      } else
+	NextSearchItem (TOKEN, GATTRmark);
+      status = CDFlib (PUT_, CDF_LEAPSECONDLASTUPDATED_, lastUpdated,
+                       NULL_);
+      if (!StatusHandlerS2C(status,cksCDF)) return FALSE;
+      break;
+    }
+    /**************************************************************************
+    *
+    **************************************************************************/
     case GATTRmark:
       if (!strcmp(item,"#GLOBALattributes")) {
 	if (mLog)
@@ -1780,13 +1854,16 @@ char *item;
       if (!strcmp(item,"="))
 	if (STRINGdataType(vDataType))
 	  NextSearchItem (BRACEDchar, VARVALUEfield);
-	else
+	else {
 	  if (vDataType == CDF_EPOCH)
 	    NextSearchItem (VAREPOCH, VARVALUEfield);
 	  else if (vDataType == CDF_EPOCH16)
 	    NextSearchItem (VAREPOCH16, VARVALUEfield);
+	  else if (vDataType == CDF_TIME_TT2000)
+	    NextSearchItem (VARTT2000, VARVALUEfield);
 	  else
 	    NextSearchItem (TOKEN, VARVALUEfield);
+        }
       else {
 	ItemError1 ("=", item);
 	return FALSE;
@@ -1841,7 +1918,7 @@ char *item;
       if (!PriorTo ("2.6.0", (Int32) version, (Int32) release,
                     (Int32) increment)) {
         zVar = BOO(zSection,TRUE,(zMode > zMODEoff));
-        status = CDFlib (PUT_, VAR_COMPRESSION(zVar), compression,
+	status = CDFlib (PUT_, VAR_COMPRESSION(zVar), compression,
                                                       compressionParms,
                          NULL_);
         if (!StatusHandlerS2C(status,cmpVAR)) return FALSE; 
@@ -1943,6 +2020,8 @@ char *item;
 	  NextSearchItem (VAREPOCH, VARPADVALUEfield);
         else if (vDataType == CDF_EPOCH16)
           NextSearchItem (VAREPOCH16, VARPADVALUEfield);
+        else if (vDataType == CDF_TIME_TT2000)
+          NextSearchItem (VARTT2000, VARPADVALUEfield);
         else if (vDataType == CDF_CHAR || vDataType == CDF_UCHAR)
           NextSearchItem (VARCHAR, VARPADVALUEfield);
         else
@@ -1953,6 +2032,8 @@ char *item;
             NextSearchItem (VAREPOCH, VARPADVALUEfield);
           else if (vDataType == CDF_EPOCH16)
             NextSearchItem (VAREPOCH16, VARPADVALUEfield);
+          else if (vDataType == CDF_TIME_TT2000)
+            NextSearchItem (VARTT2000, VARPADVALUEfield);
           else if (vDataType == CDF_CHAR || vDataType == CDF_UCHAR)
             NextSearchItem (VARCHAR, VARPADVALUEfield);
           else
@@ -2007,7 +2088,7 @@ char *item;
         len = (int) strlen (valuesB);
         if ((len-1) < vNumElems) {
           int ix;
-          for (ix = (len-1); ix < vNumElems; ++ix) *(tmp+ix) = (char) ' ';
+          for (ix = (len-1); ix < vNumElems; ++ix) *(tmp+ix) = (char) '\0';
         }
         zVar = BOO(zSection,TRUE,(zMode > zMODEoff));
         status = CDFlib (PUT_, VAR_PADVALUE(zVar), tmp,
@@ -2058,7 +2139,7 @@ char *item;
 	    if (mLog) {
 	      char tempS[MAX_MSG_LEN+1];
 	      WriteOut (stdout,
-			"WARNING: Blank padding value for variable `");
+			"WARNING: Nul padding value for variable `");
 	      WriteOut (stdout, varName);
 	      WriteOut (stdout, "'\n");
 	      snprintf (tempS, (size_t) sizeof(tempS),
@@ -2068,7 +2149,7 @@ char *item;
 	    nBytes = (size_t) (vNumElems + 1);
 	    value = cdf_AllocateMemory (nBytes, FatalError);
 	    strcpyX (value, item, (size_t) vNumElems);
-	    BlankPadRight (value, (int) (vNumElems - len));
+	    NulPadRight (value, (int) (vNumElems - len));
 	  }
 	  else
 	    value = item;
@@ -2135,6 +2216,10 @@ enum nextItemENUM item;
       epochCount = 0;
       epoch2[0] = NUL;
       break;
+    case VARTT2000:
+      epochCount = 0;
+      tt2000[0] = NUL;
+      break;
     case VARCHAR:
       charCount = 0;
       charStr[0] = NUL;
@@ -2461,147 +2546,147 @@ char **argV[];
       * Ok.
       ************************************************************************/
       case OKin: {
-		int n;
-		/**********************************************************************
-		* Get the value of each control.
-		**********************************************************************/
-		GetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
-		GetIText ((Handle) controlHs[SKTTEXTin], sktText);
-		GetIText ((Handle) controlHs[CACHEin], cacheText);
-		logMsg = GetCtlValue (controlHs[LOGin]);
-		deleteCDF = GetCtlValue (controlHs[DELETEin]);
-		reportInfos = GetCtlValue (controlHs[INFOin]);
-		reportWarns = GetCtlValue (controlHs[WARNin]);
-		reportErrors = GetCtlValue (controlHs[ERRORin]);
-		negToPos = GetCtlValue (controlHs[NEGZin]);
-		useFillval = GetCtlValue (controlHs[FILLin]);
-		dispStats = GetCtlValue (controlHs[STATSin]);
-		for (zMode = 0; zMode < 3; zMode++) {
-		   if (GetCtlValue(controlHs[ZMODEinBASE+zMode])) break;
-		}
-		/**********************************************************************
-		* Build argc/argv.
-		**********************************************************************/
-		*argC = 10 + BOO(NULpString(CDFtext),0,2) +
-				     BOO(NULpString(sktText),0,1) +
-				     BOO(NULpString(cacheText),0,2);
-		*argV = (char **) cdf_AllocateMemory ((size_t) (*argC * sizeof(char *)),
-					  FatalError);
-		n = 0;
-		MAKEstrARGv (argV, n, pgmName)
-		if (!NULpString(CDFtext)) {
-		  MAKEstrARGv (argV, n, "-cdf")
-		  PtoCstr (CDFtext);
-		  MAKEstrARGv (argV, n, (char *) CDFtext)
-		  CtoPstr ((char *) CDFtext);
-		}
-		MAKEbooARGv (argV, n, logMsg, "-log", "-nolog")
-		MAKEbooARGv (argV, n, deleteCDF, "-delete", "-nodelete")
-		MAKEbooARGv (argV, n, negToPos, "-neg2posfp0", "-noneg2posfp0")
-		MAKEbooARGv (argV, n, useFillval, "-fillval", "-nofillval")
-		MAKEbooARGv (argV, n, dispStats, "-statistics", "-nostatistics")
-		MAKEstrARGv (argV, n, "-report")
-		MAKEstrARGv (argV, n, StatusCodeReportOptions(reportErrors,
-						      reportWarns,
-						      reportInfos))
-		MAKEstrARGv (argV, n, "-zmode")
-		switch (zMode) {
-		  case zMODEoff:
-		    MAKEstrARGv (argV, n, "0")
-		    break;
-		  case zMODEon1:
-		    MAKEstrARGv (argV, n, "1")
-		    break;
-		  case zMODEon2:
-		    MAKEstrARGv (argV, n, "2")
-		    break;
-		}
-		if (!NULpString(cacheText)) {
-		  MAKEstrARGv (argV, n, "-cache")
-		  PtoCstr (cacheText);
-		  MAKEstrARGv (argV, n, (char *) cacheText)
-		  CtoPstr ((char *) cacheText);
-		}
-		if (!NULpString(sktText)) {
-		  PtoCstr (sktText);
-		  MAKEstrARGv (argV, n, (char *) sktText)
-		  CtoPstr ((char *) sktText);
-		}
-		/**********************************************************************
-		* Close the dialog and return.
-		**********************************************************************/
-		CloseDialog (dialogP);
+	int n;
+	/**********************************************************************
+	* Get the value of each control.
+	**********************************************************************/
+	GetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
+	GetIText ((Handle) controlHs[SKTTEXTin], sktText);
+	GetIText ((Handle) controlHs[CACHEin], cacheText);
+	logMsg = GetCtlValue (controlHs[LOGin]);
+	deleteCDF = GetCtlValue (controlHs[DELETEin]);
+	reportInfos = GetCtlValue (controlHs[INFOin]);
+	reportWarns = GetCtlValue (controlHs[WARNin]);
+	reportErrors = GetCtlValue (controlHs[ERRORin]);
+	negToPos = GetCtlValue (controlHs[NEGZin]);
+	useFillval = GetCtlValue (controlHs[FILLin]);
+	dispStats = GetCtlValue (controlHs[STATSin]);
+	for (zMode = 0; zMode < 3; zMode++) {
+	   if (GetCtlValue(controlHs[ZMODEinBASE+zMode])) break;
+	}
+	/**********************************************************************
+	* Build argc/argv.
+	**********************************************************************/
+	*argC = 10 + BOO(NULpString(CDFtext),0,2) +
+			     BOO(NULpString(sktText),0,1) +
+			     BOO(NULpString(cacheText),0,2);
+	*argV = (char **) cdf_AllocateMemory ((size_t) (*argC * sizeof(char *)),
+				  FatalError);
+	n = 0;
+	MAKEstrARGv (argV, n, pgmName)
+	if (!NULpString(CDFtext)) {
+	  MAKEstrARGv (argV, n, "-cdf")
+	  PtoCstr (CDFtext);
+	  MAKEstrARGv (argV, n, (char *) CDFtext)
+	  CtoPstr ((char *) CDFtext);
+	}
+	MAKEbooARGv (argV, n, logMsg, "-log", "-nolog")
+	MAKEbooARGv (argV, n, deleteCDF, "-delete", "-nodelete")
+	MAKEbooARGv (argV, n, negToPos, "-neg2posfp0", "-noneg2posfp0")
+	MAKEbooARGv (argV, n, useFillval, "-fillval", "-nofillval")
+	MAKEbooARGv (argV, n, dispStats, "-statistics", "-nostatistics")
+	MAKEstrARGv (argV, n, "-report")
+	MAKEstrARGv (argV, n, StatusCodeReportOptions(reportErrors,
+					      reportWarns,
+					      reportInfos))
+	MAKEstrARGv (argV, n, "-zmode")
+	switch (zMode) {
+	  case zMODEoff:
+	    MAKEstrARGv (argV, n, "0")
+	    break;
+	  case zMODEon1:
+	    MAKEstrARGv (argV, n, "1")
+	    break;
+	  case zMODEon2:
+	    MAKEstrARGv (argV, n, "2")
+	    break;
+	}
+	if (!NULpString(cacheText)) {
+	  MAKEstrARGv (argV, n, "-cache")
+	  PtoCstr (cacheText);
+	  MAKEstrARGv (argV, n, (char *) cacheText)
+	  CtoPstr ((char *) cacheText);
+	}
+	if (!NULpString(sktText)) {
+	  PtoCstr (sktText);
+	  MAKEstrARGv (argV, n, (char *) sktText)
+	  CtoPstr ((char *) sktText);
+	}
+	/**********************************************************************
+	* Close the dialog and return.
+	**********************************************************************/
+	CloseDialog (dialogP);
 #ifdef __MWERKS__
         DisposeRoutineDescriptor (FilterDialogQOPsoUPP);
-		DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
+	DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
 #endif
-		return TRUE;
+	return TRUE;
       }
       /************************************************************************
       * Help.
       ************************************************************************/
       case HELPin: {
-		int n;
-		*argC = 1;
-		*argV = (char **) cdf_AllocateMemory ((size_t) (*argC * sizeof(char *)),
-					  FatalError);
-		n = 0;
-		MAKEstrARGv (argV, n, pgmName)
-		CloseDialog (dialogP);
+	int n;
+	*argC = 1;
+	*argV = (char **) cdf_AllocateMemory ((size_t) (*argC * sizeof(char *)),
+				  FatalError);
+	n = 0;
+	MAKEstrARGv (argV, n, pgmName)
+	CloseDialog (dialogP);
 #ifdef __MWERKS__
         DisposeRoutineDescriptor (FilterDialogQOPsoUPP);
-		DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
+	DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
 #endif
-		return TRUE;
+	return TRUE;
       }
       /************************************************************************
       * Cancel.
       ************************************************************************/
       case CANCELin:
-		CloseDialog (dialogP);
+	CloseDialog (dialogP);
 #ifdef __MWERKS__
         DisposeRoutineDescriptor (FilterDialogQOPsoUPP);
-		DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
+	DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
 #endif
-		return FALSE;
+	return FALSE;
       /************************************************************************
       * Select skeleton CDF specification.
       *     The cursor is set because `StandardPutFile' leaves the cursor as
       * an iBeam (instead of returning it to what it was).
       ************************************************************************/
       case CDFSELECTin: {
-		StandardFileReply CDFreply;
-		char CDFpath[DU_MAX_PATH_LEN+1], prompt[] = "Enter CDF name:";
-		StandardPutFile (CtoPstr(prompt), CtoPstr(""), &CDFreply);
-		if (CDFreply.sfGood && !CDFreply.sfIsFolder && !CDFreply.sfIsVolume) {
-		  BuildMacPath (&CDFreply.sfFile, CDFpath, TRUE);
-		  CDFtext[0] = strlen (CDFpath);
-		  strcpyX ((char *) &CDFtext[1], CDFpath, 255);
-		  SetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
-		}
-		SetCursor (&(qd.arrow));
-		break;
+	StandardFileReply CDFreply;
+	char CDFpath[DU_MAX_PATH_LEN+1], prompt[] = "Enter CDF name:";
+	StandardPutFile (CtoPstr(prompt), CtoPstr(""), &CDFreply);
+	if (CDFreply.sfGood && !CDFreply.sfIsFolder && !CDFreply.sfIsVolume) {
+	  BuildMacPath (&CDFreply.sfFile, CDFpath, TRUE);
+	  CDFtext[0] = strlen (CDFpath);
+	  strcpyX ((char *) &CDFtext[1], CDFpath, 255);
+	  SetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
+	}
+	SetCursor (&(qd.arrow));
+	break;
       }
       /************************************************************************
       * Select skeleton table.
       ************************************************************************/
       case SKTSELECTin: {
-		StandardFileReply sktReply;
-		char sktPath[DU_MAX_PATH_LEN+1];
+	StandardFileReply sktReply;
+	char sktPath[DU_MAX_PATH_LEN+1];
 #ifndef __MWERKS__
-		StandardGetFile (FilterForSKTs, -1, NULL, &sktReply);
+	StandardGetFile (FilterForSKTs, -1, NULL, &sktReply);
 #else
-		FilterForSKTsUPP = NewFileFilterProc((ProcPtr) FilterForSKTs);
-		StandardGetFile (FilterForSKTsUPP, -1, NULL, &sktReply);
-		DisposeRoutineDescriptor (FilterForSKTsUPP);
+	FilterForSKTsUPP = NewFileFilterProc((ProcPtr) FilterForSKTs);
+	StandardGetFile (FilterForSKTsUPP, -1, NULL, &sktReply);
+	DisposeRoutineDescriptor (FilterForSKTsUPP);
 #endif
-		if (sktReply.sfGood && !sktReply.sfIsFolder && !sktReply.sfIsVolume) {
-		  BuildMacPath (&sktReply.sfFile, sktPath, TRUE);
-		  sktText[0] = strlen (sktPath);
-		  strcpyX ((char *) &sktText[1], sktPath, 255);
-		  SetIText ((Handle) controlHs[SKTTEXTin], sktText);
-		}
-		break;
+	if (sktReply.sfGood && !sktReply.sfIsFolder && !sktReply.sfIsVolume) {
+	  BuildMacPath (&sktReply.sfFile, sktPath, TRUE);
+	  sktText[0] = strlen (sktPath);
+	  strcpyX ((char *) &sktText[1], sktPath, 255);
+	  SetIText ((Handle) controlHs[SKTTEXTin], sktText);
+	}
+	break;
       }
       /************************************************************************
       * Check boxes.
@@ -2614,17 +2699,17 @@ char **argV[];
       case NEGZin:
       case FILLin:
       case STATSin:
-		SetCtlValue (controlHs[itemN], BOO(GetCtlValue(controlHs[itemN]),0,1));
-		break;
+	SetCtlValue (controlHs[itemN], BOO(GetCtlValue(controlHs[itemN]),0,1));
+	break;
       /************************************************************************
       * Radio buttons.
       ************************************************************************/
       case ZMODEinBASE+0:
       case ZMODEinBASE+1:
       case ZMODEinBASE+2:
-		for (i = 0; i < 3; i++) SetCtlValue (controlHs[ZMODEinBASE+i], 0);
-		SetCtlValue (controlHs[itemN], 1);
-		break;
+	for (i = 0; i < 3; i++) SetCtlValue (controlHs[ZMODEinBASE+i], 0);
+	SetCtlValue (controlHs[itemN], 1);
+	break;
       }
   }
 }
